@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: ChunkStack.cpp 824 2007-07-27 20:41:59Z bengardner $
+ * $Id: ChunkStack.cpp 1158 2008-04-12 20:51:38Z bengardner $
  */
 
 #include "ChunkStack.h"
@@ -34,10 +34,6 @@ void ChunkStack::Set(const ChunkStack& cs)
    for (int idx = 0; idx < cs.m_len; idx++)
    {
       Push(cs.m_cse[idx].m_pc, cs.m_cse[idx].m_seqnum);
-      if (cs.m_cse[idx].m_trailer != NULL)
-      {
-         SetTopTrailer(cs.m_cse[idx].m_trailer);
-      }
    }
    m_seqnum = cs.m_seqnum;
 }
@@ -51,14 +47,6 @@ const ChunkStack::Entry *ChunkStack::Top() const
    return(NULL);
 }
 
-void ChunkStack::SetTopTrailer(chunk_t *pc)
-{
-   if (m_len > 0)
-   {
-      m_cse[m_len - 1].m_trailer = pc;
-   }
-}
-
 const ChunkStack::Entry *ChunkStack::Get(int idx) const
 {
    if ((idx < m_len) && (idx >= 0))
@@ -68,6 +56,15 @@ const ChunkStack::Entry *ChunkStack::Get(int idx) const
    return(NULL);
 }
 
+chunk_t *ChunkStack::GetChunk(int idx) const
+{
+   if ((idx < m_len) && (idx >= 0))
+   {
+      return(m_cse[idx].m_pc);
+   }
+   return(NULL);
+}
+
 chunk_t *ChunkStack::Pop()
 {
    if (m_len > 0)
@@ -86,7 +83,6 @@ void ChunkStack::Push(chunk_t *pc, int seqnum)
    }
    m_cse[m_len].m_pc      = pc;
    m_cse[m_len].m_seqnum  = seqnum;
-   m_cse[m_len].m_trailer = NULL;
    m_len++;
    if (m_seqnum < seqnum)
    {
@@ -123,8 +119,7 @@ void ChunkStack::Zap(int idx)
    if ((idx < m_len) && (idx >= 0))
    {
       assert(m_cse != NULL);
-      m_cse[idx].m_pc      = NULL;
-      m_cse[idx].m_trailer = NULL;
+      m_cse[idx].m_pc = NULL;
    }
 }
 
@@ -142,9 +137,8 @@ void ChunkStack::Collapse()
       assert(m_cse != NULL);
       if (m_cse[idx].m_pc != NULL)
       {
-         m_cse[m_len].m_pc      = m_cse[idx].m_pc;
-         m_cse[m_len].m_trailer = m_cse[idx].m_trailer;
-         m_cse[m_len].m_seqnum  = m_cse[idx].m_seqnum;
+         m_cse[m_len].m_pc     = m_cse[idx].m_pc;
+         m_cse[m_len].m_seqnum = m_cse[idx].m_seqnum;
          m_len++;
       }
    }
