@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: align.cpp 932 2007-09-25 19:04:17Z bengardner $
+ * $Id: align.cpp 1178 2008-05-09 20:46:04Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -16,13 +16,15 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 
 
 static chunk_t *align_var_def_brace(chunk_t *pc, int span);
 chunk_t *align_trailing_comments(chunk_t *start);
 static void align_init_brace(chunk_t *start);
 static void align_func_params();
+static void align_func_proto(int span);
+static void align_oc_msg_spec(int span);
 
 
 static void align_typedefs(int span);
@@ -158,7 +160,7 @@ static void align_stack(ChunkStack& cs, int col, bool align_single, log_sev_t se
  * @param max_col    pointer to the column variable
  * @param extra_pad  extra padding
  */
-static void align_add(ChunkStack& cs, chunk_t *pc, int& max_col, int min_pad)
+static void align_add(ChunkStack& cs, chunk_t *pc, int& max_col, int min_pad, bool squeeze)
 {
    chunk_t *prev;
    int     min_col;
@@ -166,20 +168,26 @@ static void align_add(ChunkStack& cs, chunk_t *pc, int& max_col, int min_pad)
    prev = chunk_get_prev(pc);
    if ((prev == NULL) || chunk_is_newline(prev))
    {
-      min_col = pc->column;
+      min_col = squeeze ? 1 : pc->column;
       LOG_FMT(LALADD, "%s: pc->col=%d max_col=%d min_pad=%d min_col=%d\n",
               __func__, pc->column, max_col, min_pad, min_col);
    }
    else
    {
-      min_col = prev->column + prev->len + 1 + min_pad;
-      if (min_pad > 0)
+      if (prev->type == CT_COMMENT_MULTI)
       {
-         min_col--;
+         min_col = prev->orig_col_end + min_pad;
       }
-      if (min_col < pc->column)
+      else
       {
-         min_col = pc->column;
+         min_col = prev->column + prev->len + min_pad;
+      }
+      if (!squeeze)
+      {
+         if (min_col < pc->column)
+         {
+            min_col = pc->column;
+         }
       }
       LOG_FMT(LALADD, "%s: pc->col=%d max_col=%d min_pad=%d min_col=%d prev->col=%d\n",
               __func__, pc->column, max_col, min_pad, min_col, prev->column);
@@ -197,6 +205,36 @@ static void align_add(ChunkStack& cs, chunk_t *pc, int& max_col, int min_pad)
    }
 }
 
+void quick_align_again(void)
+{
+   chunk_t *pc;
+   chunk_t *tmp;
+   AlignStack as;
+
+   LOG_FMT(LALAGAIN, "%s\n", __func__);
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
+   {
+      if ((pc->align.next != NULL) && (pc->flags & PCF_ALIGN_START))
+      {
+         as.Start(100, 0);
+         as.m_right_align = pc->align.right_align;
+         as.m_star_style  = (AlignStack::StarStyle)pc->align.star_style;
+         as.m_amp_style   = (AlignStack::StarStyle)pc->align.amp_style;
+         as.m_gap         = pc->align.gap;
+
+         LOG_FMT(LALAGAIN, "   [%.*s:%d]", pc->len, pc->str, pc->orig_line);
+         as.Add(pc);
+         for (tmp = pc->align.next; tmp != NULL; tmp = tmp->align.next)
+         {
+            as.Add(tmp);
+            LOG_FMT(LALAGAIN, " => [%.*s:%d]", tmp->len, tmp->str, tmp->orig_line);
+         }
+         LOG_FMT(LALAGAIN, "\n");
+         as.End();
+      }
+   }
+}
+
 void align_all(void)
 {
    if (cpd.settings[UO_align_typedef_span].n > 0)
@@ -223,55 +261,55 @@ void align_all(void)
    }
 
    /* Align function prototypes */
-   if (cpd.settings[UO_align_func_proto_span].n > 0)
+   if ((cpd.settings[UO_align_func_proto_span].n > 0) &&
+       !cpd.settings[UO_align_mix_var_proto].b)
    {
       align_func_proto(cpd.settings[UO_align_func_proto_span].n);
    }
 
+   /* Align function prototypes */
+   if (cpd.settings[UO_align_oc_msg_spec_span].n > 0)
+   {
+      align_oc_msg_spec(cpd.settings[UO_align_oc_msg_spec_span].n);
+   }
+
    /* Align variable defs in function prototypes */
    if (cpd.settings[UO_align_func_params].b)
    {
       align_func_params();
    }
+
+   /* Just in case something was aligned out of order... do it again */
+   quick_align_again();
 }
 
 /**
  * Aligns all function prototypes in the file.
  */
-void align_func_proto(int span)
+static void align_oc_msg_spec(int span)
 {
    chunk_t    *pc;
-   int        max_col  = 0;
-   int        span_cnt = 0;
-   ChunkStack cs;    /* TODO: grab from a cached list */
+   AlignStack as;
 
-   pc = chunk_get_head();
-   while (pc != NULL)
+   LOG_FMT(LALIGN, "%s\n", __func__);
+   as.Start(span, 0);
+
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
    {
       if (chunk_is_newline(pc))
       {
-         if (cs.Len() > 0)
-         {
-            span_cnt += pc->nl_count;
-            if (span_cnt > span)
-            {
-               align_stack(cs, max_col, false, LALPROTO);
-               max_col = 0;
-            }
-         }
+         as.NewLines(pc->nl_count);
       }
-      else if (pc->type == CT_FUNC_PROTO)
+      else if (pc->type == CT_OC_MSG_SPEC)
       {
-         align_add(cs, pc, max_col, 0);
-         span_cnt = 0;
+         as.Add(pc);
       }
-      pc = chunk_get_next(pc);
    }
-   align_stack(cs, max_col, false, LALPROTO);
+   as.End();
 }
 
 /**
- * Aligns all backslah-newline combos in the file.
+ * Aligns all backslash-newline combos in the file.
  * This should be done LAST.
  */
 void align_backslash_newline(void)
@@ -314,11 +352,13 @@ void align_right_comments(void)
 void align_struct_initializers(void)
 {
    chunk_t *pc;
+   chunk_t *prev;
 
    pc = chunk_get_head();
    while (pc != NULL)
    {
-      if ((pc->parent_type == CT_ASSIGN) &&
+      prev = chunk_get_prev_ncnl(pc);
+      if ((prev != NULL) && (prev->type == CT_ASSIGN) &&
           ((pc->type == CT_BRACE_OPEN) ||
            ((cpd.lang_flags & LANG_D) && (pc->type == CT_SQUARE_OPEN))))
       {
@@ -635,6 +675,51 @@ static void align_func_params()
    }
 }
 
+/**
+ * Aligns all function prototypes in the file.
+ */
+static void align_func_proto(int span)
+{
+   chunk_t    *pc;
+   bool       look_bro = false;
+   AlignStack as;
+   AlignStack as_br;
+
+   LOG_FMT(LALIGN, "%s\n", __func__);
+   as.Start(span, 0);
+   as.m_gap = cpd.settings[UO_align_func_proto_gap].n;
+
+   as_br.Start(span, 0);
+   as_br.m_gap = cpd.settings[UO_align_single_line_brace_gap].n;
+
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
+   {
+      if (chunk_is_newline(pc))
+      {
+         look_bro = false;
+         as.NewLines(pc->nl_count);
+         as_br.NewLines(pc->nl_count);
+      }
+      else if ((pc->type == CT_FUNC_PROTO) ||
+               ((pc->type == CT_FUNC_DEF) &&
+                cpd.settings[UO_align_single_line_func].b))
+      {
+         as.Add(pc);
+         look_bro = (pc->type == CT_FUNC_DEF) &&
+                    cpd.settings[UO_align_single_line_brace].b;
+      }
+      else if (look_bro &&
+               (pc->type == CT_BRACE_OPEN) &&
+               (pc->flags & PCF_ONE_LINER))
+      {
+         as_br.Add(pc);
+         look_bro = false;
+      }
+   }
+   as.End();
+   as_br.End();
+}
+
 /**
  * Scan everything at the current level until the close brace and find the
  * variable def align column.  Also aligns bit-colons, but that assumes that
@@ -644,10 +729,16 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
 {
    chunk_t    *pc;
    chunk_t    *next;
+   chunk_t    *prev;
    int        align_mask = PCF_IN_FCN_DEF | PCF_VAR_1ST;
    int        myspan     = span;
    int        mythresh   = 0;
-   AlignStack as;
+   int        mygap      = 0;
+   AlignStack as;    /* var/proto/def */
+   AlignStack as_bc; /* bit-colon */
+   AlignStack as_br; /* one-liner brace open */
+   bool       fp_active   = cpd.settings[UO_align_mix_var_proto].b;
+   bool       fp_look_bro = false;
 
 
    if (start == NULL)
@@ -661,33 +752,47 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
    {
       myspan   = cpd.settings[UO_align_var_struct_span].n;
       mythresh = cpd.settings[UO_align_var_struct_thresh].n;
+      mygap    = cpd.settings[UO_align_var_struct_gap].n;
    }
    else
    {
       mythresh = cpd.settings[UO_align_var_def_thresh].n;
+      mygap    = cpd.settings[UO_align_var_def_gap].n;
    }
 
    /* can't be any variable definitions in a "= {" block */
-   if (start->parent_type == CT_ASSIGN)
+   prev = chunk_get_prev_ncnl(start);
+   if ((prev != NULL) && (prev->type == CT_ASSIGN))
    {
+      LOG_FMT(LAVDB, "%s: start=%.*s [%s] on line %d (abort due to assign)\n", __func__,
+              start->len, start->str, get_token_name(start->type), start->orig_line);
+
       pc = chunk_get_next_type(start, CT_BRACE_CLOSE, start->level);
       return(chunk_get_next_ncnl(pc));
    }
 
+   LOG_FMT(LAVDB, "%s: start=%.*s [%s] on line %d\n", __func__,
+           start->len, start->str, get_token_name(start->type), start->orig_line);
 
    if (!cpd.settings[UO_align_var_def_inline].b)
    {
       align_mask |= PCF_VAR_INLINE;
    }
 
-   //LOG_FMT(LSYS, "%s: start=[%.*s] line=%d col=%d level=%d\n",
-   //        __func__, start->len, start->str, start->orig_line, start->orig_col, start->level);
-
+   /* Set up the var/proto/def aligner */
    as.Start(myspan, mythresh);
-
+   as.m_gap        = mygap;
    as.m_star_style = (AlignStack::StarStyle)cpd.settings[UO_align_var_def_star_style].n;
    as.m_amp_style  = (AlignStack::StarStyle)cpd.settings[UO_align_var_def_amp_style].n;
 
+   /* Set up the bit colon aligner */
+   as_bc.Start(myspan, 0);
+   as_bc.m_gap = cpd.settings[UO_align_var_def_colon_gap].n;
+
+   /* Set up the brace open aligner */
+   as_br.Start(myspan, mythresh);
+   as_br.m_gap = cpd.settings[UO_align_single_line_brace_gap].n;
+
    bool did_this_line = false;
    pc = chunk_get_next_ncnl(start);
    while ((pc != NULL) && ((pc->level >= start->level) || (pc->level == 0)))
@@ -697,11 +802,35 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
          if (pc->nl_count > 0)
          {
             as.NewLines(pc->nl_count);
+            as_bc.NewLines(pc->nl_count);
+            as_br.NewLines(pc->nl_count);
          }
          pc = chunk_get_next(pc);
          continue;
       }
 
+      if (fp_active)
+      {
+         if ((pc->type == CT_FUNC_PROTO) ||
+             ((pc->type == CT_FUNC_DEF) &&
+              cpd.settings[UO_align_single_line_func].b))
+         {
+            LOG_FMT(LAVDB, "    add=[%.*s] line=%d col=%d level=%d\n",
+                    pc->len, pc->str, pc->orig_line, pc->orig_col, pc->level);
+
+            as.Add(pc);
+            fp_look_bro = (pc->type == CT_FUNC_DEF) &&
+                          cpd.settings[UO_align_single_line_brace].b;
+         }
+         else if (fp_look_bro &&
+                  (pc->type == CT_BRACE_OPEN) &&
+                  (pc->flags & PCF_ONE_LINER))
+         {
+            as_br.Add(pc);
+            fp_look_bro = false;
+         }
+      }
+
       /* process nested braces */
       if (pc->type == CT_BRACE_OPEN)
       {
@@ -718,8 +847,11 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
 
       if (chunk_is_newline(pc))
       {
+         fp_look_bro   = false;
          did_this_line = false;
          as.NewLines(pc->nl_count);
+         as_bc.NewLines(pc->nl_count);
+         as_br.NewLines(pc->nl_count);
       }
 
       /* don't align stuff inside parens/squares/angles */
@@ -736,15 +868,18 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
       {
          if (!did_this_line)
          {
-            //LOG_FMT(LSYS, "    add=[%.*s] line=%d col=%d level=%d\n",
-            //        pc->len, pc->str, pc->orig_line, pc->orig_col, pc->level);
+            LOG_FMT(LAVDB, "    add=[%.*s] line=%d col=%d level=%d\n",
+                    pc->len, pc->str, pc->orig_line, pc->orig_col, pc->level);
 
             as.Add(pc);
 
-            next = chunk_get_next_nc(pc);
-            if (next->type == CT_BIT_COLON)
+            if (cpd.settings[UO_align_var_def_colon].b)
             {
-               as.AddTrailer(next);
+               next = chunk_get_next_nc(pc);
+               if (next->type == CT_BIT_COLON)
+               {
+                  as_bc.Add(next);
+               }
             }
          }
          did_this_line = true;
@@ -753,6 +888,8 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
    }
 
    as.End();
+   as_bc.End();
+   as_br.End();
 
    return(pc);
 }
@@ -781,7 +918,7 @@ chunk_t *align_nl_cont(chunk_t *start)
    {
       if (pc->type == CT_NL_CONT)
       {
-         align_add(cs, pc, max_col, 0);
+         align_add(cs, pc, max_col, 1, true);
       }
       pc = chunk_get_next(pc);
    }
@@ -798,6 +935,35 @@ chunk_t *align_nl_cont(chunk_t *start)
    return(pc);
 }
 
+enum CmtAlignType
+{
+   CAT_REGULAR,
+   CAT_BRACE,
+   CAT_ENDIF,
+};
+
+static CmtAlignType get_comment_align_type(chunk_t *cmt)
+{
+   chunk_t      *prev;
+   CmtAlignType cmt_type = CAT_REGULAR;
+
+   if (!cpd.settings[UO_align_right_cmt_mix].b &&
+       ((prev = chunk_get_prev(cmt)) != NULL))
+   {
+      if ((prev->type == CT_PP_ENDIF) ||
+          (prev->type == CT_PP_ELSE) ||
+          (prev->type == CT_BRACE_CLOSE))
+      {
+         /* REVISIT: someone may want this configurable */
+         if ((cmt->column - (prev->column + prev->len)) < 3)
+         {
+            cmt_type = (prev->type == CT_PP_ENDIF) ? CAT_ENDIF : CAT_BRACE;
+         }
+      }
+   }
+   return(cmt_type);
+}
+
 /**
  * For a series of lines ending in a comment, align them.
  * The series ends when more than align_right_cmt_span newlines are found.
@@ -807,18 +973,26 @@ chunk_t *align_nl_cont(chunk_t *start)
  */
 chunk_t *align_trailing_comments(chunk_t *start)
 {
-   int        max_col  = 0;
-   chunk_t    *pc      = start;
-   int        nl_count = 0;
-   ChunkStack cs;
+   int          max_col  = 0;
+   chunk_t      *pc      = start;
+   int          nl_count = 0;
+   ChunkStack   cs;
+   CmtAlignType cmt_type_start, cmt_type_cur;
+
+   cmt_type_start = get_comment_align_type(pc);
 
    /* Find the max column */
    while ((pc != NULL) && (nl_count < cpd.settings[UO_align_right_cmt_span].n))
    {
       if ((pc->flags & PCF_RIGHT_COMMENT) != 0)
       {
-         align_add(cs, pc, max_col, 0);
-         nl_count = 0;
+         cmt_type_cur = get_comment_align_type(pc);
+
+         if (cmt_type_cur == cmt_type_start)
+         {
+            align_add(cs, pc, max_col, 1, false);
+            nl_count = 0;
+         }
       }
       if (chunk_is_newline(pc))
       {
