@@ -7,7 +7,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: brace_cleanup.cpp 987 2007-11-02 18:53:04Z bengardner $
+ * $Id: brace_cleanup.cpp 1176 2008-05-09 20:38:15Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -18,7 +18,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 
 
 static chunk_t *insert_vbrace(chunk_t *pc, bool after,
@@ -159,7 +159,8 @@ void brace_cleanup(void)
        * Also need to pass in the initial '#' to close out any virtual braces.
        */
       if (!chunk_is_comment(pc) && !chunk_is_newline(pc) &&
-          ((cpd.in_preproc == CT_PP_DEFINE) || (cpd.in_preproc == CT_NONE)))
+          ((cpd.in_preproc == CT_PP_DEFINE) ||
+           (cpd.in_preproc == CT_NONE)))
       {
          cpd.consumed = false;
          parse_cleanup(&frm, pc);
@@ -199,6 +200,26 @@ static bool maybe_while_of_do(chunk_t *pc)
    return(false);
 }
 
+static void push_fmr_pse(struct parse_frame *frm, chunk_t *pc,
+                         brstage_e stage, const char *logtext)
+{
+   if (frm->pse_tos < (int)ARRAY_SIZE(frm->pse))
+   {
+      frm->pse_tos++;
+      frm->pse[frm->pse_tos].type  = pc->type;
+      frm->pse[frm->pse_tos].stage = stage;
+      frm->pse[frm->pse_tos].pc    = pc;
+
+      print_stack(LBCSPUSH, logtext, frm, pc);
+   }
+   else
+   {
+      LOG_FMT(LWARN, "%s:%d Error: Frame stack overflow,  Unable to properly process this file.\n",
+              cpd.filename, cpd.line_number);
+      cpd.error_count++;
+   }
+}
+
 /**
  * At the heart of this algorithm are two stacks.
  * There is the Paren Stack (PS) and the Frame stack.
@@ -360,9 +381,10 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
          if ((frm->pse[frm->pse_tos].type != CT_NONE) &&
              (frm->pse[frm->pse_tos].type != CT_PP_DEFINE))
          {
-            LOG_FMT(LWARN, "%s:%d Error: Unexpected '%.*s' for '%s'\n",
+            LOG_FMT(LWARN, "%s:%d Error: Unexpected '%.*s' for '%s', which was on line %d\n",
                     cpd.filename, pc->orig_line, pc->len, pc->str,
-                    get_token_name(frm->pse[frm->pse_tos].type));
+                    get_token_name(frm->pse[frm->pse_tos].pc->type),
+                    frm->pse[frm->pse_tos].pc->orig_line);
             print_stack(LBCSPOP, "=Error  ", frm, pc);
             cpd.error_count++;
          }
@@ -374,7 +396,7 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
          /* Copy the parent, update the paren/brace levels */
          pc->parent_type = frm->pse[frm->pse_tos].parent;
          frm->level--;
-         if ((pc->type == CT_BRACE_CLOSE)  ||
+         if ((pc->type == CT_BRACE_CLOSE) ||
              (pc->type == CT_VBRACE_CLOSE) ||
              (pc->type == CT_MACRO_CLOSE))
          {
@@ -509,13 +531,9 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
       {
          frm->brace_level++;
       }
-      frm->pse_tos++;
-      frm->pse[frm->pse_tos].type   = pc->type;
-      frm->pse[frm->pse_tos].stage  = BS_NONE;
+      push_fmr_pse(frm, pc, BS_NONE, "+Open   ");
       frm->pse[frm->pse_tos].parent = parent;
       pc->parent_type = parent;
-
-      print_stack(LBCSPUSH, "+Open   ", frm, pc);
    }
 
    pattern_class patcls = get_token_pattern_class(pc->type);
@@ -523,43 +541,28 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
    /** Create a stack entry for complex statments IF/DO/FOR/WHILE/SWITCH */
    if (patcls == PATCLS_BRACED)
    {
-      frm->pse_tos++;
-      frm->pse[frm->pse_tos].type  = pc->type;
-      frm->pse[frm->pse_tos].stage = (pc->type == CT_DO) ? BS_BRACE_DO : BS_BRACE2;
-
-      print_stack(LBCSPUSH, "+ComplexBraced", frm, pc);
+      push_fmr_pse(frm, pc,
+                   (pc->type == CT_DO) ? BS_BRACE_DO : BS_BRACE2,
+                   "+ComplexBraced");
    }
    else if (patcls == PATCLS_PBRACED)
    {
-      frm->pse_tos++;
+      brstage_e bs = BS_PAREN1;
+
       if ((pc->type == CT_WHILE) && maybe_while_of_do(pc))
       {
          pc->type = CT_WHILE_OF_DO;
-         frm->pse[frm->pse_tos].stage = BS_WOD_PAREN;
+         bs       = BS_WOD_PAREN;
       }
-      else
-      {
-         frm->pse[frm->pse_tos].stage = BS_PAREN1;
-      }
-      frm->pse[frm->pse_tos].type = pc->type;
-
-      print_stack(LBCSPUSH, "+ComplexParenBraced", frm, pc);
+      push_fmr_pse(frm, pc, bs, "+ComplexParenBraced");
    }
    else if (patcls == PATCLS_OPBRACED)
    {
-      frm->pse_tos++;
-      frm->pse[frm->pse_tos].type  = pc->type;
-      frm->pse[frm->pse_tos].stage = BS_OP_PAREN1;
-
-      print_stack(LBCSPUSH, "+ComplexOpParenBraced", frm, pc);
+      push_fmr_pse(frm, pc, BS_OP_PAREN1, "+ComplexOpParenBraced");
    }
    else if (patcls == PATCLS_ELSE)
    {
-      frm->pse_tos++;
-      frm->pse[frm->pse_tos].type  = pc->type;
-      frm->pse[frm->pse_tos].stage = BS_ELSEIF;
-
-      print_stack(LBCSPUSH, "+ComplexElse", frm, pc);
+      push_fmr_pse(frm, pc, BS_ELSEIF, "+ComplexElse");
    }
 
    /* Mark simple statement/expression starts
@@ -722,13 +725,9 @@ static bool check_complex_statements(struct parse_frame *frm, chunk_t *pc)
       frm->level++;
       frm->brace_level++;
 
-      frm->pse_tos++;
-      frm->pse[frm->pse_tos].type   = CT_VBRACE_OPEN;
-      frm->pse[frm->pse_tos].stage  = BS_NONE;
+      push_fmr_pse(frm, vbrace, BS_NONE, "+VBrace ");
       frm->pse[frm->pse_tos].parent = parent;
 
-      print_stack(LBCSPUSH, "+VBrace ", frm, pc);
-
       /* update the level of pc */
       pc->level       = frm->level;
       pc->brace_level = frm->brace_level;
@@ -886,6 +885,11 @@ static chunk_t *insert_vbrace(chunk_t *pc, bool after,
    else
    {
       ref = chunk_get_prev(pc);
+      if ((ref->flags & PCF_IN_PREPROC) == 0)
+      {
+         chunk.flags &= ~PCF_IN_PREPROC;
+      }
+
       while (chunk_is_newline(ref) || chunk_is_comment(ref))
       {
          ref->level++;
@@ -894,9 +898,20 @@ static chunk_t *insert_vbrace(chunk_t *pc, bool after,
       }
 
       /* Don't back into a preprocessor */
-      if ((ref->flags & PCF_IN_PREPROC) != (pc->flags & PCF_IN_PREPROC))
+      if (((pc->flags & PCF_IN_PREPROC) == 0) &&
+          ((ref->flags & PCF_IN_PREPROC) != 0))
       {
-         ref = chunk_get_next(ref);
+         if (ref->type == CT_PREPROC_BODY)
+         {
+            do
+            {
+               ref = chunk_get_prev(ref);
+            } while ((ref != NULL) && ((ref->flags & PCF_IN_PREPROC) != 0));
+         }
+         else
+         {
+            ref = chunk_get_next(ref);
+         }
       }
 
       chunk.orig_line = ref->orig_line;
