@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: braces.cpp 960 2007-10-13 01:53:50Z bengardner $
+ * $Id: braces.cpp 1181 2008-05-09 21:21:19Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -14,13 +14,14 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 
 
 static void convert_vbrace_to_brace(void);
 static void examine_braces(void);
 static void examine_brace(chunk_t *bopen);
 static void remove_brace(chunk_t *pc);
+static void move_case_break();
 
 
 void do_braces(void)
@@ -81,6 +82,11 @@ void do_braces(void)
          }
       }
    }
+
+   if (cpd.settings[UO_mod_move_case_break].b)
+   {
+      move_case_break();
+   }
 }
 
 /**
@@ -257,14 +263,19 @@ static void convert_vbrace_to_brace(void)
 {
    chunk_t *pc;
    chunk_t *tmp;
+   chunk_t *vbc;
+   bool    in_preproc;
 
+   /* Find every vbrace open */
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnl(pc))
    {
-      if ((pc->type != CT_VBRACE_OPEN) && (pc->type != CT_VBRACE_CLOSE))
+      if (pc->type != CT_VBRACE_OPEN)
       {
          continue;
       }
 
+      in_preproc = (pc->flags & PCF_IN_PREPROC) != 0;
+
       if ((((pc->parent_type == CT_IF) ||
             (pc->parent_type == CT_ELSE) ||
             (pc->parent_type == CT_ELSEIF)) &&
@@ -282,33 +293,197 @@ static void convert_vbrace_to_brace(void)
           ((pc->parent_type == CT_FUNC_DEF) &&
            ((cpd.settings[UO_mod_full_brace_function].a & AV_ADD) != 0)))
       {
-         if (pc->type == CT_VBRACE_OPEN)
+         /* Find the matching vbrace close */
+         vbc = NULL;
+         tmp = pc;
+         while ((tmp = chunk_get_next(tmp)) != NULL)
          {
-            pc->type = CT_BRACE_OPEN;
-            pc->len  = 1;
-            pc->str  = "{";
-         }
-         else
-         {
-            pc->type = CT_BRACE_CLOSE;
-            pc->len  = 1;
-            pc->str  = "}";
-
-            /* If the next chunk is a comment, followed by a newline, then
-             * move the brace after the newline and add another newline after
-             * the close brace.
-             */
-            tmp = chunk_get_next(pc);
-            if (chunk_is_comment(tmp))
+            if (in_preproc && ((tmp->flags & PCF_IN_PREPROC) == 0))
             {
-               tmp = chunk_get_next(tmp);
-               if (chunk_is_newline(tmp))
-               {
-                  chunk_move_after(pc, tmp);
-                  newline_add_after(pc);
-               }
+               /* Can't leave a preprocessor */
+               break;
+            }
+            if ((pc->brace_level == tmp->brace_level) &&
+                (tmp->type == CT_VBRACE_CLOSE) &&
+                (pc->parent_type == tmp->parent_type) &&
+                ((tmp->flags & PCF_IN_PREPROC) == (pc->flags & PCF_IN_PREPROC)))
+            {
+               vbc = tmp;
+               break;
+            }
+         }
+         if (vbc == NULL)
+         {
+            continue;
+         }
+
+         pc->type = CT_BRACE_OPEN;
+         pc->len  = 1;
+         pc->str  = "{";
+
+         vbc->type = CT_BRACE_CLOSE;
+         vbc->len  = 1;
+         vbc->str  = "}";
+
+         /* If the next chunk is a comment, followed by a newline, then
+          * move the brace after the newline and add another newline after
+          * the close brace.
+          */
+         tmp = chunk_get_next(vbc);
+         if (chunk_is_comment(tmp))
+         {
+            tmp = chunk_get_next(tmp);
+            if (chunk_is_newline(tmp))
+            {
+               chunk_move_after(vbc, tmp);
+               newline_add_after(vbc);
             }
          }
       }
    }
 }
+
+/**
+ * Adds a comment after the ref chunk
+ * Returns the added chunk or NULL
+ */
+chunk_t *insert_comment_after(chunk_t *ref, c_token_t cmt_type,
+                              int cmt_len, const char *cmt_text)
+{
+   chunk_t new_cmt;
+   char    *txt;
+   int     txt_len;
+
+   if (cmt_len <= 0)
+   {
+      cmt_len = strlen(cmt_text);
+   }
+   txt_len = cmt_len + 8; /* 8 is big enough for all types */
+
+   memset(&new_cmt, 0, sizeof(new_cmt));
+   new_cmt.flags = (ref->flags & PCF_COPY_FLAGS) | PCF_OWN_STR;
+   new_cmt.type  = cmt_type;
+
+   /* allocate memory for the string */
+   txt = new char[txt_len];
+   if (txt == NULL)
+   {
+      return(NULL);
+   }
+
+   new_cmt.str = txt;
+   if (cmt_type == CT_COMMENT_CPP)
+   {
+      new_cmt.len = snprintf(txt, txt_len, "// %.*s", cmt_len, cmt_text);
+   }
+   else
+   {
+      new_cmt.len = snprintf(txt, txt_len, "/* %.*s */", cmt_len, cmt_text);
+   }
+   /* TODO: expand comment type to cover other comment styles? */
+
+   new_cmt.column   = ref->column + ref->len + 1;
+   new_cmt.orig_col = new_cmt.column;
+
+   return(chunk_add_after(&new_cmt, ref));
+}
+
+void add_long_closebrace_comment(void)
+{
+   chunk_t *pc;
+   chunk_t *tmp;
+   chunk_t *br_open;
+   chunk_t *br_close;
+   chunk_t *fcn_pc = NULL;
+   chunk_t *sw_pc  = NULL;
+   int     nl_count;
+
+   pc = chunk_get_head();
+   while ((pc = chunk_get_next_ncnl(pc)) != NULL)
+   {
+      if (pc->type == CT_FUNC_DEF)
+      {
+         fcn_pc = pc;
+      }
+      else if (pc->type == CT_SWITCH)
+      {
+         /* kinda pointless, since it always has the text "switch" */
+         sw_pc = pc;
+      }
+      if ((pc->type != CT_BRACE_OPEN) || ((pc->flags & PCF_IN_PREPROC) != 0))
+      {
+         continue;
+      }
+
+      br_open  = pc;
+      nl_count = 0;
+
+      tmp = pc;
+      while ((tmp = chunk_get_next(tmp)) != NULL)
+      {
+         if (chunk_is_newline(tmp))
+         {
+            nl_count += tmp->nl_count;
+         }
+         else if ((tmp->level == br_open->level) &&
+                  (tmp->type == CT_BRACE_CLOSE))
+         {
+            br_close = tmp;
+
+            //LOG_FMT(LSYS, "found brace pair on lines %d and %d, nl_count=%d\n",
+            //        br_open->orig_line, br_close->orig_line, nl_count);
+
+            /* Found the matching close brace - make sure a newline is next */
+            tmp = chunk_get_next(tmp);
+            if ((tmp == NULL) || chunk_is_newline(tmp))
+            {
+               int     nl_min  = 0;
+               chunk_t *tag_pc = NULL;
+
+               if (br_open->parent_type == CT_SWITCH)
+               {
+                  nl_min = cpd.settings[UO_mod_add_long_switch_closebrace_comment].n;
+                  tag_pc = sw_pc;
+               }
+               else if (br_open->parent_type == CT_FUNC_DEF)
+               {
+                  nl_min = cpd.settings[UO_mod_add_long_function_closebrace_comment].n;
+                  tag_pc = fcn_pc;
+               }
+
+               if ((nl_min > 0) && (nl_count >= nl_min) && (tag_pc != NULL))
+               {
+                  /* TODO: determine the added comment style */
+
+                  /* Add a comment after the close brace */
+                  insert_comment_after(br_close, CT_COMMENT,
+                                       tag_pc->len, tag_pc->str);
+               }
+            }
+            break;
+         }
+      }
+   }
+}
+
+static void move_case_break()
+{
+   chunk_t *pc;
+   chunk_t *prev = NULL;
+
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnl(pc))
+   {
+      if ((pc->type == CT_BREAK) &&
+          (prev != NULL) &&
+          (prev->type == CT_BRACE_CLOSE) &&
+          (prev->parent_type == CT_CASE))
+      {
+         if (chunk_is_newline(chunk_get_prev(pc)) &&
+             chunk_is_newline(chunk_get_prev(prev)))
+         {
+            chunk_swap_lines(prev, pc);
+         }
+      }
+      prev = pc;
+   }
+}
