@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: chunk_list.cpp 965 2007-10-15 00:18:47Z bengardner $
+ * $Id: chunk_list.cpp 1042 2007-11-30 22:27:44Z bengardner $
  */
 
 #include "chunk_list.h"
@@ -121,6 +121,11 @@ chunk_t *chunk_add_before(const chunk_t *pc_in, chunk_t *ref)
 void chunk_del(chunk_t *pc)
 {
    g_cl.Pop(pc);
+   if ((pc->flags & PCF_OWN_STR) && (pc->str != NULL))
+   {
+      delete[] (char *)pc->str;
+      pc->str = NULL;
+   }
    delete pc;
 }
 
@@ -444,6 +449,104 @@ void chunk_swap(chunk_t *pc1, chunk_t *pc2)
    g_cl.Swap(pc1, pc2);
 }
 
+/**
+ * Finds the first chunk on the line that pc is on.
+ * This just backs up until a newline or NULL is hit.
+ *
+ * given: [ a - b - c - n1 - d - e - n2 ]
+ * input: [ a | b | c | n1 ] => a
+ * input: [ d | e | n2 ]     => d
+ */
+chunk_t *chunk_first_on_line(chunk_t *pc)
+{
+   chunk_t *first = pc;
+
+   while (((pc = chunk_get_prev(pc)) != NULL) && !chunk_is_newline(pc))
+   {
+      first = pc;
+   }
+
+   return(first);
+}
+
+/**
+ * Swaps two lines that are started with the specified chunks.
+ *
+ * @param pc1  The first chunk of line 1
+ * @param pc2  The first chunk of line 2
+ */
+void chunk_swap_lines(chunk_t *pc1, chunk_t *pc2)
+{
+   chunk_t *ref2;
+   chunk_t *tmp;
+
+   pc1 = chunk_first_on_line(pc1);
+   pc2 = chunk_first_on_line(pc2);
+
+   if ((pc1 == NULL) || (pc2 == NULL) || (pc1 == pc2))
+   {
+      return;
+   }
+
+   /**
+    * Example start:
+    * ? - start1 - a1 - b1 - nl1 - ? - ref2 - start2 - a2 - b2 - nl2 - ?
+    *      ^- pc1                              ^- pc2
+    */
+   ref2 = chunk_get_prev(pc2);
+
+   /* Move the line started at pc2 before pc1 */
+   while ((pc2 != NULL) && !chunk_is_newline(pc2))
+   {
+      tmp = chunk_get_next(pc2);
+      g_cl.Pop(pc2);
+      g_cl.AddBefore(pc2, pc1);
+      pc2 = tmp;
+   }
+
+   /**
+    * Should now be:
+    * ? - start2 - a2 - b2 - start1 - a1 - b1 - nl1 - ? - ref2 - nl2 - ?
+    *                         ^- pc1                              ^- pc2
+    */
+
+   /* Now move the line started at pc1 after ref2 */
+   while ((pc1 != NULL) && !chunk_is_newline(pc1))
+   {
+      tmp = chunk_get_next(pc1);
+      g_cl.Pop(pc1);
+      if (ref2 != NULL)
+      {
+         g_cl.AddAfter(pc1, ref2);
+      }
+      else
+      {
+         g_cl.AddHead(pc1);
+      }
+      ref2 = pc1;
+      pc1  = tmp;
+   }
+
+   /**
+    * Should now be:
+    * ? - start2 - a2 - b2 - nl1 - ? - ref2 - start1 - a1 - b1 - nl2 - ?
+    *                         ^- pc1                              ^- pc2
+    */
+
+   /* pc1 and pc2 should be the newlines for their lines.
+    * swap the chunks and the nl_count so that the spacing remains the same.
+    */
+   if ((pc1 != NULL) && (pc2 != NULL))
+   {
+      int nl_count = pc1->nl_count;
+
+      pc1->nl_count = pc2->nl_count;
+      pc2->nl_count = nl_count;
+
+      chunk_swap(pc1, pc2);
+   }
+}
+
 /**
  * Gets the next non-vbrace chunk
  */
