@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: combine.cpp 994 2007-11-05 20:56:12Z bengardner $
+ * $Id: combine.cpp 1183 2008-05-30 00:02:58Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -16,7 +16,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 #include <cassert>
 
 static void fix_fcn_def_params(chunk_t *pc);
@@ -28,6 +28,7 @@ static chunk_t *fix_var_def(chunk_t *pc);
 static void mark_function(chunk_t *pc);
 static void mark_struct_union_body(chunk_t *start);
 static chunk_t *mark_variable_definition(chunk_t *start);
+
 static void mark_define_expressions(void);
 static void process_returns(void);
 static chunk_t *process_return(chunk_t *pc);
@@ -38,6 +39,10 @@ static void mark_cpp_constructor(chunk_t *pc);
 static void mark_lvalue(chunk_t *pc);
 static void mark_template_func(chunk_t *pc, chunk_t *pc_next);
 static void mark_exec_sql(chunk_t *pc);
+static void handle_oc_class(chunk_t *pc);
+static void handle_oc_message(chunk_t *pc);
+static void handle_template(chunk_t *pc);
+
 
 void make_type(chunk_t *pc)
 {
@@ -76,6 +81,7 @@ static chunk_t *flag_parens(chunk_t *po, UINT32 flags,
    {
       LOG_FMT(LERR, "%s: no match for [%.*s] at  [%d:%d]\n",
               __func__, po->len, po->str, po->orig_line, po->orig_col);
+      return(NULL);
    }
 
    if ((paren_close != NULL) && (po != paren_close))
@@ -130,6 +136,46 @@ chunk_t *set_paren_parent(chunk_t *start, c_token_t parent)
    return(chunk_get_next_ncnl(end));
 }
 
+/* Scan backwards to see if we might be on a type declaration */
+static bool chunk_ends_type(chunk_t *pc)
+{
+    bool ret = false;
+
+    for (/* nada */; pc != NULL; pc = chunk_get_prev_ncnl(pc))
+    {
+       LOG_FMT(LFTYPE, "%s: [%s] %.*s flags %x on line %d, col %d\n",
+               __func__, get_token_name(pc->type), pc->len, pc->str,
+               pc->flags, pc->orig_line, pc->orig_col);
+
+       if ((pc->type == CT_WORD) ||
+           (pc->type == CT_TYPE) ||
+           (pc->type == CT_STRUCT) ||
+           (pc->type == CT_DC_MEMBER) ||
+           (pc->type == CT_QUALIFIER))
+       {
+          continue;
+       }
+
+       if (chunk_is_semicolon(pc) ||
+           (pc->type == CT_BRACE_OPEN) ||
+           (pc->type == CT_BRACE_CLOSE))
+       {
+          ret = true;
+       }
+       break;
+    }
+
+    if (pc == NULL)
+    {
+        /* first token */
+        ret = true;
+    }
+
+    LOG_FMT(LFTYPE, "%s verdict: %s\n", __func__, ret ? "yes" : "no");
+
+    return(ret);
+}
+
 /**
  * Change CT_INCDEC_AFTER + WORD to CT_INCDEC_BEFORE
  * Change number/word + CT_ADDR to CT_ARITH
@@ -210,7 +256,7 @@ void fix_symbols(void)
 
             for (tmp = chunk_get_prev_ncnl(pc); tmp != NULL; tmp = chunk_get_prev_ncnl(tmp))
             {
-               if ((tmp->type == CT_SEMICOLON) ||
+               if (chunk_is_semicolon(tmp) ||
                    (tmp->type == CT_BRACE_OPEN) ||
                    (tmp->type == CT_VBRACE_OPEN))
                {
@@ -231,9 +277,20 @@ void fix_symbols(void)
                tmp->parent_type = pc->type;
             }
          }
-
       } /* paren open + cast/align/delegate */
 
+      /* Objective C stuff */
+      if (cpd.lang_flags & LANG_OC)
+      {
+         /* Check for message declarations */
+         if ((pc->flags & PCF_STMT_START) &&
+             (chunk_is_str(pc, "-", 1) || chunk_is_str(pc, "+", 1)) &&
+             chunk_is_str(next, "(", 1))
+         {
+            handle_oc_message(pc);
+         }
+      }
+
       if ((pc->type == CT_ASSIGN) && (next->type == CT_SQUARE_OPEN))
       {
          set_paren_parent(next, CT_ASSIGN);
@@ -294,6 +351,20 @@ void fix_symbols(void)
          }
       }
 
+      if (pc->type == CT_EXTERN)
+      {
+         tmp = chunk_get_next_type(next, CT_BRACE_OPEN, next->level);
+         if (tmp != NULL)
+         {
+            set_paren_parent(tmp, CT_EXTERN);
+         }
+      }
+
+      if (pc->type == CT_TEMPLATE)
+      {
+         handle_template(pc);
+      }
+
       if ((pc->type == CT_WORD) &&
           (next->type == CT_ANGLE_OPEN) &&
           (next->parent_type == CT_TEMPLATE))
@@ -416,7 +487,7 @@ void fix_symbols(void)
                   set_paren_parent(tmp, pc->type);
                }
             }
-            else if ((tmp->type == CT_SEMICOLON) && (pc->type == CT_FUNC_PROTO))
+            else if (chunk_is_semicolon(tmp) && (pc->type == CT_FUNC_PROTO))
             {
                tmp->parent_type = pc->type;
             }
@@ -429,6 +500,15 @@ void fix_symbols(void)
          fix_fcn_def_params(next);
       }
 
+      if ((pc->type == CT_THROW) && (prev->type == CT_FPAREN_CLOSE))
+      {
+         pc->parent_type = prev->parent_type;
+         if (next->type == CT_PAREN_OPEN)
+         {
+            set_paren_parent(next, CT_THROW);
+         }
+      }
+
       /* Mark the braces in: "for_each_entry(xxx) { }" */
       if ((pc->type == CT_BRACE_OPEN) &&
           (prev->type == CT_FPAREN_CLOSE) &&
@@ -469,6 +549,11 @@ void fix_symbols(void)
          }
       }
 
+      if (pc->type == CT_OC_CLASS)
+      {
+         handle_oc_class(pc);
+      }
+
       if (pc->type == CT_NAMESPACE)
       {
          mark_namespace(pc);
@@ -576,10 +661,14 @@ void fix_symbols(void)
          {
             pc->type = CT_PTR_TYPE;
          }
-         else if (prev->type == CT_SIZEOF)
+         else if ((prev->type == CT_SIZEOF) || (prev->type == CT_DELETE))
          {
             pc->type = CT_DEREF;
          }
+         else if ((prev->type == CT_WORD) && chunk_ends_type(prev))
+         {
+            pc->type = CT_PTR_TYPE;
+         }
          else
          {
             pc->type = (prev->type == CT_ARITH) ? CT_DEREF : CT_ARITH;
@@ -588,17 +677,24 @@ void fix_symbols(void)
 
       if (pc->type == CT_AMP)
       {
-         pc->type = CT_ARITH;
-         if (prev->type == CT_WORD)
+         if (prev->type == CT_DELETE)
          {
-            tmp = chunk_get_prev_ncnl(prev);
-            if ((tmp->type == CT_SEMICOLON) ||
-                (tmp->type == CT_BRACE_OPEN) ||
-                (tmp->type == CT_QUALIFIER))
+            pc->type = CT_DEREF;
+         }
+         else
+         {
+            pc->type = CT_ARITH;
+            if (prev->type == CT_WORD)
             {
-               prev->type   = CT_TYPE;
-               pc->type     = CT_ADDR;
-               next->flags |= PCF_VAR_1ST;
+               tmp = chunk_get_prev_ncnl(prev);
+               if (chunk_is_semicolon(tmp) ||
+                   (tmp->type == CT_BRACE_OPEN) ||
+                   (tmp->type == CT_QUALIFIER))
+               {
+                  prev->type   = CT_TYPE;
+                  pc->type     = CT_ADDR;
+                  next->flags |= PCF_VAR_1ST;
+               }
             }
          }
       }
@@ -651,6 +747,7 @@ void fix_symbols(void)
             square_level = -1;
          }
       }
+
       /**
        * A variable definition is possible after at the start of a statement
        * that starts with: QUALIFIER, TYPE, or WORD
@@ -709,8 +806,7 @@ static void mark_lvalue(chunk_t *pc)
           (prev->type == CT_ASSIGN) ||
           (prev->type == CT_COMMA) ||
           (prev->type == CT_BOOL) ||
-          (prev->type == CT_SEMICOLON) ||
-          (prev->type == CT_VSEMICOLON) ||
+          chunk_is_semicolon(prev) ||
           chunk_is_str(prev, "(", 1) ||
           chunk_is_str(prev, "{", 1) ||
           chunk_is_str(prev, "[", 1) ||
@@ -954,7 +1050,7 @@ static bool is_ucase_str(const char *str, int len)
 {
    while (len-- > 0)
    {
-      if (toupper(*str) != *str)
+      if (unc_toupper(*str) != *str)
       {
          return(false);
       }
@@ -1260,6 +1356,11 @@ static void fix_enum_struct_union(chunk_t *pc)
       return;
    }
 
+   if ((cpd.lang_flags & LANG_D) && !chunk_is_semicolon(next))
+   {
+      next = pawn_add_vsemi_after(chunk_get_prev_ncnl(next));
+   }
+
    /* We are either pointing to a ';' or a variable */
    while ((next != NULL) && !chunk_is_semicolon(next) &&
           (next->type != CT_ASSIGN) &&
@@ -1268,7 +1369,7 @@ static void fix_enum_struct_union(chunk_t *pc)
       if (next->type == CT_WORD)
       {
          next->flags |= flags;
-         flags       &= ~PCF_VAR_1ST;/* clear the first flag for the next items */
+         flags       &= ~PCF_VAR_1ST; /* clear the first flag for the next items */
       }
 
       if (next->type == CT_STAR)
@@ -1346,8 +1447,8 @@ static void fix_typedef(chunk_t *start)
                continue;
             }
 
-            prev->parent_type = CT_TYPEDEF;
-            open_paren = chunk_get_prev_type(prev, c_token_t(prev->type - 1), prev->level);
+            prev->parent_type       = CT_TYPEDEF;
+            open_paren              = chunk_get_prev_type(prev, c_token_t(prev->type - 1), prev->level);
             open_paren->parent_type = CT_TYPEDEF;
 
             flag_parens(next, 0, CT_FPAREN_OPEN, CT_TYPEDEF, false);
@@ -1412,9 +1513,12 @@ static void fix_typedef(chunk_t *start)
       }
    }
 
-   LOG_FMT(LTYPEDEF, "%s: %s typedef [%.*s] on line %d\n",
-           __func__, get_token_name(tag), the_type->len, the_type->str, the_type->orig_line);
-   the_type->flags |= PCF_ANCHOR;
+   if (the_type != NULL)
+   {
+      LOG_FMT(LTYPEDEF, "%s: %s typedef [%.*s] on line %d\n",
+              __func__, get_token_name(tag), the_type->len, the_type->str, the_type->orig_line);
+      the_type->flags |= PCF_ANCHOR;
+   }
 }
 
 /**
@@ -1439,7 +1543,9 @@ void combine_labels(void)
    /* unlikely that the file will start with a label... */
    while (next != NULL)
    {
-      if ((next->type == CT_CLASS) || (next->type == CT_TEMPLATE))
+      if ((next->type == CT_CLASS) ||
+          (next->type == CT_OC_CLASS) ||
+          (next->type == CT_TEMPLATE))
       {
          hit_class = true;
       }
@@ -1683,15 +1789,15 @@ static chunk_t *skip_to_next_statement(chunk_t *pc)
  */
 static chunk_t *fix_var_def(chunk_t *start)
 {
-   chunk_t *pc = start;
-   chunk_t *before_end;
-   chunk_t *end;
-   int     type_count = 0;
+   chunk_t    *pc = start;
+   chunk_t    *end;
+   chunk_t    *tmp_pc;
+   ChunkStack cs;
+   int        idx, ref_idx;
 
    LOG_FMT(LFVD, "%s: top[%d]", __func__, pc->orig_line);
 
    /* Scan for words and types and stars oh my! */
-   before_end = pc;
    while ((pc != NULL) &&
           ((pc->type == CT_TYPE) ||
            (pc->type == CT_WORD) ||
@@ -1702,13 +1808,7 @@ static chunk_t *fix_var_def(chunk_t *start)
            chunk_is_star(pc)))
    {
       LOG_FMT(LFVD, " %.*s[%s]", pc->len, pc->str, get_token_name(pc->type));
-      type_count++;
-      before_end = pc;
-      if ((pc->type == CT_DC_MEMBER) ||
-          (pc->type == CT_MEMBER))
-      {
-         type_count = 0;
-      }
+      cs.Push(pc);
       pc = chunk_get_next_ncnl(pc);
 
       /* Skip templates and attributes */
@@ -1724,35 +1824,66 @@ static chunk_t *fix_var_def(chunk_t *start)
       return(NULL);
    }
 
-   /* A single word can only be a type if followed by a function */
-   if ((type_count == 1) && (end->type != CT_FUNC_DEF))
+   /* Function defs are handled elsewhere */
+   if ((cs.Len() <= 1) ||
+       (end->type == CT_FUNC_DEF) ||
+       (end->type == CT_FUNC_PROTO) ||
+       (end->type == CT_FUNC_CLASS))
    {
-      /* Scan to the next semicolon or brace open or close */
       return(skip_to_next_statement(end));
    }
 
-   /* Everything before a function def or class func is a type */
-   if ((end->type == CT_FUNC_DEF) || (end->type == CT_FUNC_CLASS))
+   /* ref_idx points to the alignable part of the var def */
+   ref_idx = cs.Len() - 1;
+
+   /* Check for the '::' stuff: "char *Engine::name" */
+   if ((cs.Len() >= 3) &&
+       ((cs.Get(cs.Len() - 2)->m_pc->type == CT_MEMBER) ||
+        (cs.Get(cs.Len() - 2)->m_pc->type == CT_DC_MEMBER)))
    {
-      for (pc = start; pc != end; pc = chunk_get_next_ncnl(pc))
+      idx = cs.Len() - 2;
+      while (idx > 0)
       {
-         make_type(pc);
+         tmp_pc = cs.Get(idx)->m_pc;
+         if ((tmp_pc->type != CT_DC_MEMBER) &&
+             (tmp_pc->type != CT_MEMBER))
+         {
+            break;
+         }
+         idx--;
+         tmp_pc = cs.Get(idx)->m_pc;
+         if ((tmp_pc->type != CT_WORD) &&
+             (tmp_pc->type != CT_TYPE))
+         {
+            break;
+         }
+         make_type(tmp_pc);
+         idx--;
       }
+      ref_idx = idx + 1;
+   }
+   tmp_pc = cs.Get(ref_idx)->m_pc;
+   LOG_FMT(LFVD, " ref_idx(%d) => %.*s\n", ref_idx, tmp_pc->len, tmp_pc->str);
+
+   /* No type part found! */
+   if (ref_idx <= 0)
+   {
       return(skip_to_next_statement(end));
    }
 
-   LOG_FMT(LFVD, "%s:%d TYPE : ", __func__, start->orig_line);
-   for (pc = start; pc != before_end; pc = chunk_get_next_ncnl(pc))
+   LOG_FMT(LFVD2, "%s:%d TYPE : ", __func__, start->orig_line);
+   for (idx = 0; idx < cs.Len() - 1; idx++)
    {
-      make_type(pc);
-      LOG_FMT(LFVD, " %.*s[%s]", pc->len, pc->str, get_token_name(pc->type));
+      tmp_pc = cs.Get(idx)->m_pc;
+      make_type(tmp_pc);
+      LOG_FMT(LFVD2, " %.*s[%s]", tmp_pc->len, tmp_pc->str, get_token_name(tmp_pc->type));
    }
-   LOG_FMT(LFVD, "\n");
+   LOG_FMT(LFVD2, "\n");
 
    /**
     * OK we have two or more items, mark types up to the end.
     */
-   mark_variable_definition(before_end);
+   mark_variable_definition(cs.Get(cs.Len() - 1)->m_pc);
    if (end->type == CT_COMMA)
    {
       return(chunk_get_next_ncnl(end));
@@ -1864,6 +1995,8 @@ static bool can_be_full_param(chunk_t *start, chunk_t *end)
    chunk_t *pc;
    chunk_t *last;
    int     word_cnt = 0;
+   int     type_count = 0;
+   bool    ret;
 
    LOG_FMT(LFPARAM, "%s:", __func__);
 
@@ -1884,6 +2017,10 @@ static bool can_be_full_param(chunk_t *start, chunk_t *end)
           (pc->type == CT_TYPE))
       {
          word_cnt++;
+         if (pc->type == CT_TYPE)
+         {
+            type_count++;
+         }
       }
       else if ((pc->type == CT_MEMBER) ||
                (pc->type == CT_DC_MEMBER))
@@ -1924,9 +2061,11 @@ static bool can_be_full_param(chunk_t *start, chunk_t *end)
       return(true);
    }
 
+   ret = ((word_cnt >= 2) || ((word_cnt == 1) && (type_count == 1)));
+
    LOG_FMT(LFPARAM, " <== [%s] %s!\n",
-           get_token_name(pc->type), (word_cnt >= 2) ? "Yup" : "Unlikely");
-   return(word_cnt >= 2);
+           get_token_name(pc->type), ret ? "Yup" : "Unlikely");
+   return(ret);
 }
 
 /**
@@ -1991,6 +2130,13 @@ static void mark_function(chunk_t *pc)
    paren_open  = chunk_get_next_str(pc, "(", 1, pc->level);
    paren_close = chunk_get_next_str(paren_open, ")", 1, pc->level);
 
+   if ((paren_open == NULL) || (paren_close == NULL))
+   {
+      LOG_FMT(LFCN, "No parens found for [%.*s] on line %d col %d\n",
+              pc->len, pc->str, pc->orig_line, pc->orig_col);
+      return;
+   }
+
    /**
     * This part detects either chained function calls or a function ptr definition.
     * MYTYPE (*func)(void);
@@ -2253,11 +2399,16 @@ static void mark_function(chunk_t *pc)
 
    /* Scan tokens until we hit a brace open (def) or semicolon (proto) */
    tmp = paren_close;
-   while (((tmp = chunk_get_next_ncnl(tmp)) != NULL) &&
-          (tmp->level >= pc->level))
+   while ((tmp = chunk_get_next_ncnl(tmp)) != NULL)
    {
       /* Only care about brace or semi on the same level */
-      if (tmp->level == pc->level)
+      if (tmp->level < pc->level)
+      {
+         /* No semicolon - probably a function call? */
+         pc->type = CT_FUNC_CALL;
+         break;
+      }
+      else if (tmp->level == pc->level)
       {
          if (tmp->type == CT_BRACE_OPEN)
          {
@@ -2299,7 +2450,7 @@ static void mark_function(chunk_t *pc)
        (pc->type == CT_FUNC_PROTO) &&
        (pc->parent_type != CT_OPERATOR))
    {
-      LOG_FMT(LFPARAM, "%s :: checking '%.*s' for contructor variable %s %s\n",
+      LOG_FMT(LFPARAM, "%s :: checking '%.*s' for constructor variable %s %s\n",
               __func__, pc->len, pc->str,
               get_token_name(paren_open->type),
               get_token_name(paren_close->type));
@@ -2317,7 +2468,7 @@ static void mark_function(chunk_t *pc)
       while (tmp != paren_close)
       {
          tmp2 = chunk_get_next_ncnl(tmp);
-         if (tmp->type == CT_COMMA)
+         if ((tmp->type == CT_COMMA) && (tmp->level == (paren_open->level + 1)))
          {
             if (!can_be_full_param(ref, tmp))
             {
@@ -2350,6 +2501,7 @@ static void mark_function(chunk_t *pc)
             chunk_t *p_op = chunk_get_prev_type(pc, CT_BRACE_OPEN, pc->brace_level - 1);
             if ((p_op != NULL) &&
                 (p_op->parent_type != CT_CLASS) &&
+                (p_op->parent_type != CT_STRUCT) &&
                 (p_op->parent_type != CT_NAMESPACE))
             {
                pc->type = CT_FUNC_CTOR_VAR;
@@ -2399,7 +2551,8 @@ static void mark_function(chunk_t *pc)
    {
       bool on_first = true;
       tmp = chunk_get_next_ncnl(paren_close);
-      while ((tmp != NULL) && (tmp->type != CT_BRACE_OPEN))
+      while ((tmp != NULL) &&
+             (tmp->type != CT_BRACE_OPEN))
       {
          //LOG_FMT(LSYS, "%s: set parent to FUNC_DEF on line %d: [%.*s]\n", __func__, tmp->orig_line, tmp->len, tmp->str);
          tmp->parent_type = CT_FUNC_DEF;
@@ -2453,7 +2606,7 @@ static void mark_cpp_constructor(chunk_t *pc)
    /* Scan until the brace open, mark everything */
    tmp = paren_open;
    while ((tmp != NULL) && (tmp->type != CT_BRACE_OPEN) &&
-          (tmp->type != CT_SEMICOLON))
+          !chunk_is_semicolon(tmp))
    {
       tmp->flags |= PCF_IN_CONST_ARGS;
       tmp         = chunk_get_next_ncnl(tmp);
@@ -2761,6 +2914,64 @@ static void mark_define_expressions(void)
    }
 }
 
+/**
+ * We are on the 'template' C++ keyword.
+ * What follows should be the following:
+ *
+ * template <class identifier> function_declaration;
+ * template <typename identifier> function_declaration;
+ * template <class identifier> class class_declaration;
+ * template <typename identifier> class class_declaration;
+ *
+ * Change the 'class' inside the <> to CT_TYPE.
+ * Set the parent to the class after the <> to CT_TEMPLATE.
+ * Set the parent of the semicolon to CT_TEMPLATE.
+ */
+static void handle_template(chunk_t *pc)
+{
+   chunk_t *tmp;
+   int     level;
+
+   tmp = chunk_get_next_ncnl(pc);
+   if (tmp->type != CT_ANGLE_OPEN)
+   {
+      return;
+   }
+   tmp->parent_type = CT_TEMPLATE;
+
+   level = tmp->level;
+
+   while ((tmp = chunk_get_next(tmp)) != NULL)
+   {
+      if ((tmp->type == CT_CLASS) ||
+          (tmp->type == CT_STRUCT))
+      {
+         tmp->type = CT_TYPE;
+      }
+      else if ((tmp->type == CT_ANGLE_CLOSE) && (tmp->level == level))
+      {
+         tmp->parent_type = CT_TEMPLATE;
+         break;
+      }
+   }
+   if (tmp != NULL)
+   {
+      tmp = chunk_get_next_ncnl(tmp);
+      if ((tmp != NULL) &&
+          ((tmp->type == CT_CLASS) || (tmp->type == CT_STRUCT)))
+      {
+         tmp->parent_type = CT_TEMPLATE;
+
+         /* REVISTI: This may be a bit risky - might need to track the { }; */
+         tmp = chunk_get_next_type(tmp, CT_SEMICOLON, tmp->level);
+         if (tmp != NULL)
+         {
+            tmp->parent_type = CT_TEMPLATE;
+         }
+      }
+   }
+}
+
 /**
  * We are on a word followed by a angle open which is part of a template.
  * If the angle close is followed by a open paren, then we are on a template
@@ -2916,3 +3127,126 @@ chunk_t *skip_attribute_prev(chunk_t *fp_close)
    return(fp_close);
 }
 
+/**
+ * Process an ObjC 'class'
+ * pc is the chunk after '@implementation' or '@interface'.
+ * Change colons, etc.
+ */
+static void handle_oc_class(chunk_t *pc)
+{
+   chunk_t *tmp = pc;
+
+   while ((tmp = chunk_get_next(tmp)) != NULL)
+   {
+      if ((tmp->type == CT_SEMICOLON) ||
+          (tmp->type == CT_BRACE_OPEN))
+      {
+         tmp->parent_type = CT_OC_CLASS;
+         break;
+      }
+      else if (tmp->type == CT_COLON)
+      {
+         tmp->type = CT_CLASS_COLON;
+      }
+   }
+
+   if ((tmp != NULL) && (tmp->type == CT_BRACE_OPEN))
+   {
+      tmp = chunk_get_next_type(tmp, CT_BRACE_CLOSE, tmp->level);
+      if (tmp != NULL)
+      {
+         tmp->parent_type = CT_OC_CLASS;
+      }
+   }
+}
+
+/**
+ * Process an ObjC message spec/dec
+ *
+ * Specs:
+ * -(void) foo ARGS;
+ *
+ * Decl:
+ * -(void) foo ARGS {  }
+ *
+ * ARGS is ': (type) name [name]'
+ * -(void) foo: (int) arg: {  }
+ */
+static void handle_oc_message(chunk_t *pc)
+{
+   chunk_t *tmp;
+   bool    in_paren  = false;
+   int     paren_cnt = 0;
+   c_token_t pt;
+
+   /* Figure out if this is a spec or decl */
+   tmp = pc;
+   while ((tmp = chunk_get_next(tmp)) != NULL)
+   {
+      if ((tmp->type == CT_SEMICOLON) ||
+          (tmp->type == CT_BRACE_OPEN))
+      {
+         pt = (tmp->type == CT_SEMICOLON) ? CT_OC_MSG_SPEC : CT_OC_MSG_DECL;
+         break;
+      }
+   }
+   if (tmp == NULL)
+   {
+      return;
+   }
+   pc->type        = CT_OC_SCOPE;
+   pc->parent_type = pt;
+
+   /* Mark everything */
+   tmp = pc;
+   while ((tmp = chunk_get_next(tmp)) != NULL)
+   {
+      if ((tmp->type == CT_SEMICOLON) ||
+          (tmp->type == CT_BRACE_OPEN))
+      {
+         tmp->parent_type = pt;
+         break;
+      }
+
+      if ((tmp->type == CT_PAREN_OPEN) ||
+          (tmp->type == CT_PAREN_CLOSE))
+      {
+         tmp->parent_type = pt;
+         in_paren         = (tmp->type == CT_PAREN_OPEN);
+         if (!in_paren)
+         {
+            paren_cnt++;
+         }
+      }
+      else if (tmp->type == CT_WORD)
+      {
+         if (in_paren)
+         {
+            tmp->type = CT_TYPE;
+         }
+         else if (paren_cnt == 1)
+         {
+            tmp->type = pt;
+         }
+         else
+         {
+            tmp->flags |= PCF_VAR_DEF;
+         }
+      }
+      else if (tmp->type == CT_COLON)
+      {
+         tmp->type        = CT_OC_COLON;
+         tmp->parent_type = pt;
+      }
+   }
+
+   if ((tmp != NULL) && (tmp->type == CT_BRACE_OPEN))
+   {
+      tmp = chunk_get_next_type(tmp, CT_BRACE_CLOSE, tmp->level);
+      if (tmp != NULL)
+      {
+         tmp->parent_type = pt;
+      }
+   }
+}
+
