@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: indent.cpp 997 2007-11-08 00:06:09Z bengardner $
+ * $Id: indent.cpp 1180 2008-05-09 21:03:39Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -14,7 +14,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 
 
 /**
@@ -130,6 +130,60 @@ void indent_to_column(chunk_t *pc, int column)
    reindent_line(pc, column);
 }
 
+/* Same as indent_to_column, except we can move both ways */
+void align_to_column(chunk_t *pc, int column)
+{
+   if (column == pc->column)
+   {
+      return;
+   }
+
+   int col_delta;
+   int min_col;
+
+   LOG_FMT(LINDLINE, "%s: %d] col %d on %.*s [%s] => %d\n",
+           __func__, pc->orig_line, pc->column, pc->len, pc->str,
+           get_token_name(pc->type), column);
+
+   col_delta  = column - pc->column;
+   pc->column = column;
+   min_col    = column;
+
+   do
+   {
+      chunk_t *next = chunk_get_next(pc);
+
+      if (next == NULL)
+      {
+         break;
+      }
+      min_col += space_col_align(pc, next);
+      pc = next;
+      bool is_comment = chunk_is_comment(pc);
+      bool keep = is_comment && chunk_is_single_line_comment(pc) &&
+                  cpd.settings[UO_indent_relative_single_line_comments].b;
+
+      if (is_comment && (pc->parent_type != CT_COMMENT_EMBED) && !keep)
+      {
+         pc->column = pc->orig_col;
+         if (pc->column < min_col)
+         {
+            pc->column = min_col;// + 1;
+         }
+         LOG_FMT(LINDLINE, "%s: set comment on line %d to col %d (orig %d)\n",
+                 __func__, pc->orig_line, pc->column, pc->orig_col);
+      }
+      else
+      {
+         pc->column += col_delta;
+         if (pc->column < min_col)
+         {
+            pc->column = min_col;
+         }
+      }
+   } while ((pc != NULL) && (pc->nl_count == 0));
+}
+
 /**
  * Changes the initial indent for a line to the given column
  *
@@ -151,35 +205,39 @@ void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
    }
    col_delta  = column - pc->column;
    pc->column = column;
-   min_col    = pc->column;
+   min_col    = column;
 
    do
    {
-      min_col += pc->len;
-      if (pc->flags & PCF_FORCE_SPACE)
+      chunk_t *next = chunk_get_next(pc);
+
+      if (next == NULL)
       {
-         min_col++;
+         break;
       }
-      pc = chunk_get_next(pc);
-      if (pc != NULL)
+      min_col += space_col_align(pc, next);
+      pc = next;
+
+      bool is_comment = chunk_is_comment(pc);
+      bool keep = is_comment && chunk_is_single_line_comment(pc) &&
+                  cpd.settings[UO_indent_relative_single_line_comments].b;
+
+      if (is_comment && (pc->parent_type != CT_COMMENT_EMBED) && !keep)
       {
-         if (chunk_is_comment(pc) && (pc->parent_type != CT_COMMENT_EMBED))
+         pc->column = pc->orig_col;
+         if (pc->column < min_col)
          {
-            pc->column = pc->orig_col;
-            if (pc->column < min_col)
-            {
-               pc->column = min_col + 1;
-            }
-            LOG_FMT(LINDLINE, "%s: set comment on line %d to col %d (orig %d)\n",
-                    __func__, pc->orig_line, pc->column, pc->orig_col);
+            pc->column = min_col;// + 1;
          }
-         else
+         LOG_FMT(LINDLINE, "%s: set comment on line %d to col %d (orig %d)\n",
+                 __func__, pc->orig_line, pc->column, pc->orig_col);
+      }
+      else
+      {
+         pc->column += col_delta;
+         if (pc->column < min_col)
          {
-            pc->column += col_delta;
-            if (pc->column < min_col)
-            {
-               pc->column = min_col;
-            }
+            pc->column = min_col;
          }
       }
    } while ((pc != NULL) && (pc->nl_count == 0));
@@ -276,6 +334,13 @@ static int token_indent(c_token_t type)
    }
 }
 
+#define indent_column_set(X) \
+   do { \
+      indent_column = (X); \
+       LOG_FMT(LINDENT2, "[line %d] indent_column = %d\n", \
+               __LINE__, indent_column); \
+   } while (0)
+
 /**
  * Change the top-level indentation only by changing the column member in
  * the chunk structures.
@@ -299,8 +364,8 @@ void indent_text(void)
    int                parent_token_indent = 0;
    int                xml_indent          = 0;
    bool               token_used;
-   int                sql_col             = 0;
-   int                sql_orig_col        = 0;
+   int                sql_col      = 0;
+   int                sql_orig_col = 0;
 
    memset(&frm, 0, sizeof(frm));
 
@@ -338,15 +403,6 @@ void indent_text(void)
             {
                indent_pse_pop(frm, pc);
             }
-
-            /* If we just removed an #endregion, then check to see if a
-             * PP_REGION_INDENT entry is right below it
-             */
-            if (((type == CT_PP_ENDIF) || (type == CT_PP_ELSE)) &&
-                (frm.pse[frm.pse_tos].type == CT_PP_IF_INDENT))
-            {
-               indent_pse_pop(frm, pc);
-            }
          }
       }
       else if (pc->type == CT_PREPROC)
@@ -373,7 +429,7 @@ void indent_text(void)
 
             /* Indent one level */
             frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + indent_size;
-            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab + indent_size;;
+            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab + indent_size;
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
             frm.pse[frm.pse_tos].in_preproc = false;
          }
@@ -391,7 +447,7 @@ void indent_text(void)
 
             /* Indent one level */
             frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + indent_size;
-            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab + indent_size;;
+            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab + indent_size;
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
             frm.pse[frm.pse_tos].in_preproc = false;
          }
@@ -409,7 +465,8 @@ void indent_text(void)
          else
          {
             if ((frm.pse[frm.pse_tos - 1].type == CT_PP_REGION_INDENT) ||
-                (frm.pse[frm.pse_tos - 1].type == CT_PP_IF_INDENT))
+                ((frm.pse[frm.pse_tos - 1].type == CT_PP_IF_INDENT) &&
+                 (frm.pse[frm.pse_tos].type != CT_PP_ENDIF)))
             {
                frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 2].indent;
             }
@@ -446,7 +503,6 @@ void indent_text(void)
             }
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
          }
-
       }
 
       if ((cout_col > 0) &&
@@ -500,7 +556,7 @@ void indent_text(void)
             indent_pse_pop(frm, pc);
          }
 
-         if (frm.pse[frm.pse_tos].level == pc->level)
+         if (frm.pse[frm.pse_tos].level >= pc->level)
          {
             /* process virtual braces closes (no text output) */
             if ((pc->type == CT_VBRACE_CLOSE) &&
@@ -512,17 +568,23 @@ void indent_text(void)
             }
 
             /* End any assign operations with a semicolon on the same level */
-            if ((frm.pse[frm.pse_tos].type == CT_ASSIGN) &&
+            if (((frm.pse[frm.pse_tos].type == CT_ASSIGN_NL) ||
+                 (frm.pse[frm.pse_tos].type == CT_ASSIGN)) &&
                 (chunk_is_semicolon(pc) ||
                  (pc->type == CT_COMMA) ||
                  (pc->type == CT_BRACE_OPEN) ||
+                 (pc->type == CT_SPAREN_CLOSE) ||
+                 (pc->type == CT_FPAREN_CLOSE) ||
                  ((pc->type == CT_SQUARE_OPEN) && (pc->parent_type == CT_ASSIGN))))
             {
                indent_pse_pop(frm, pc);
             }
 
             /* End any assign operations with a semicolon on the same level */
-            if ((frm.pse[frm.pse_tos].type == CT_IMPORT) && chunk_is_semicolon(pc))
+            if (chunk_is_semicolon(pc) &&
+                ((frm.pse[frm.pse_tos].type == CT_IMPORT) ||
+                 (frm.pse[frm.pse_tos].type == CT_THROW) ||
+                 (frm.pse[frm.pse_tos].type == CT_USING)))
             {
                indent_pse_pop(frm, pc);
             }
@@ -552,6 +614,15 @@ void indent_text(void)
                indent_pse_pop(frm, pc);
             }
 
+            /* a class scope is ended with another class scope or a close brace */
+            if (cpd.settings[UO_indent_access_spec_body].b &&
+                (frm.pse[frm.pse_tos].type == CT_PRIVATE) &&
+                ((pc->type == CT_BRACE_CLOSE) ||
+                 (pc->type == CT_PRIVATE)))
+            {
+               indent_pse_pop(frm, pc);
+            }
+
             /* a return is ended with a semicolon */
             if ((frm.pse[frm.pse_tos].type == CT_RETURN) &&
                 chunk_is_semicolon(pc))
@@ -581,7 +652,7 @@ void indent_text(void)
       } while (old_pse_tos > frm.pse_tos);
 
       /* Grab a copy of the current indent */
-      indent_column = frm.pse[frm.pse_tos].indent_tmp;
+      indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
 
       if (!chunk_is_newline(pc) && !chunk_is_comment(pc))
       {
@@ -622,7 +693,7 @@ void indent_text(void)
          if (frm.pse[frm.pse_tos].type == CT_BRACE_OPEN)
          {
             /* Indent the brace to match the open brace */
-            indent_column = frm.pse[frm.pse_tos].brace_indent;
+            indent_column_set(frm.pse[frm.pse_tos].brace_indent);
 
             indent_pse_pop(frm, pc);
             frm.level--;
@@ -638,7 +709,7 @@ void indent_text(void)
          frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
 
          /* Always indent on virtual braces */
-         indent_column = frm.pse[frm.pse_tos].indent_tmp;
+         indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
       }
       else if (pc->type == CT_BRACE_OPEN)
       {
@@ -671,7 +742,7 @@ void indent_text(void)
                else
                {
                   frm.pse[frm.pse_tos].indent += cpd.settings[UO_indent_brace].n;
-                  indent_column += cpd.settings[UO_indent_brace].n;
+                  indent_column_set(indent_column + cpd.settings[UO_indent_brace].n);
                }
             }
             else if (pc->parent_type == CT_CASE)
@@ -681,8 +752,8 @@ void indent_text(void)
                 * So we need to take the CASE indent, subtract off the
                 * indent_size that was added above and then add indent_case_brace.
                 */
-               indent_column = frm.pse[frm.pse_tos - 1].indent - indent_size +
-                               cpd.settings[UO_indent_case_brace].n;
+               indent_column_set(frm.pse[frm.pse_tos - 1].indent - indent_size +
+                                 cpd.settings[UO_indent_case_brace].n);
 
                /* Stuff inside the brace still needs to be indented */
                frm.pse[frm.pse_tos].indent     = indent_column + indent_size;
@@ -696,6 +767,10 @@ void indent_text(void)
             {
                frm.pse[frm.pse_tos].indent -= indent_size;
             }
+            else if ((pc->parent_type == CT_EXTERN) && !cpd.settings[UO_indent_extern].b)
+            {
+               frm.pse[frm.pse_tos].indent -= indent_size;
+            }
 
             frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
          }
@@ -703,7 +778,7 @@ void indent_text(void)
          if ((pc->flags & PCF_DONT_INDENT) != 0)
          {
             frm.pse[frm.pse_tos].indent = pc->column;
-            indent_column = pc->column;
+            indent_column_set(pc->column);
          }
          else
          {
@@ -725,7 +800,7 @@ void indent_text(void)
             /* Update the indent_column if needed */
             if (brace_indent || (parent_token_indent != 0))
             {
-               indent_column = frm.pse[frm.pse_tos].indent_tmp;
+               indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
             }
          }
 
@@ -738,7 +813,7 @@ void indent_text(void)
          {
             indent_pse_pop(frm, pc);
             frm.level--;
-            indent_column = frm.pse[frm.pse_tos].indent_tmp;
+            indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
          }
       }
       else if (pc->type == CT_SQL_BEGIN)
@@ -768,7 +843,7 @@ void indent_text(void)
       {
          if (frm.pse[frm.pse_tos].type == CT_MACRO_OPEN)
          {
-            indent_column = frm.pse[frm.pse_tos - 1].indent;
+            indent_column_set(frm.pse[frm.pse_tos - 1].indent);
          }
       }
       else if (pc->type == CT_CASE)
@@ -783,32 +858,61 @@ void indent_text(void)
          frm.pse[frm.pse_tos].indent_tab = tmp;
 
          /* Always set on case statements */
-         indent_column = frm.pse[frm.pse_tos].indent_tmp;
+         indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
+      }
+      else if (pc->type == CT_BREAK)
+      {
+         prev = chunk_get_prev_ncnl(pc);
+         if ((prev != NULL) &&
+             (prev->type == CT_BRACE_CLOSE) &&
+             (prev->parent_type == CT_CASE))
+         {
+            /* This only affects the 'break', so no need for a stack entry */
+            indent_column_set(prev->column);
+         }
       }
       else if (pc->type == CT_LABEL)
       {
          /* Labels get sent to the left or backed up */
          if (cpd.settings[UO_indent_label].n > 0)
          {
-            indent_column = cpd.settings[UO_indent_label].n;
+            indent_column_set(cpd.settings[UO_indent_label].n);
          }
          else
          {
-            indent_column = frm.pse[frm.pse_tos].indent +
-                            cpd.settings[UO_indent_label].n;
+            indent_column_set(frm.pse[frm.pse_tos].indent +
+                              cpd.settings[UO_indent_label].n);
          }
       }
       else if (pc->type == CT_PRIVATE)
       {
-         /* Labels get sent to the left or backed up */
-         if (cpd.settings[UO_indent_access_spec].n > 0)
+         if (cpd.settings[UO_indent_access_spec_body].b)
          {
-            indent_column = cpd.settings[UO_indent_access_spec].n;
+            tmp = frm.pse[frm.pse_tos].indent + indent_size;
+
+            indent_pse_push(frm, pc);
+
+            frm.pse[frm.pse_tos].indent     = tmp;
+            frm.pse[frm.pse_tos].indent_tmp = tmp - indent_size;
+            frm.pse[frm.pse_tos].indent_tab = tmp;
+
+            /* If we are indenting the body, then we must leave the access spec
+             * indented at brace level
+             */
+            indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
          }
          else
          {
-            indent_column = frm.pse[frm.pse_tos].indent +
-                            cpd.settings[UO_indent_access_spec].n;
+            /* Access spec labels get sent to the left or backed up */
+            if (cpd.settings[UO_indent_access_spec].n > 0)
+            {
+               indent_column_set(cpd.settings[UO_indent_access_spec].n);
+            }
+            else
+            {
+               indent_column_set(frm.pse[frm.pse_tos].indent +
+                                 cpd.settings[UO_indent_access_spec].n);
+            }
          }
       }
       else if (pc->type == CT_CLASS_COLON)
@@ -819,7 +923,7 @@ void indent_text(void)
          frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
          frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
 
-         indent_column = frm.pse[frm.pse_tos].indent_tmp;
+         indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
 
          if (cpd.settings[UO_indent_class_colon].b)
          {
@@ -849,11 +953,44 @@ void indent_text(void)
          indent_pse_push(frm, pc);
          frm.pse[frm.pse_tos].indent = pc->column + pc->len;
 
-         if (cpd.settings[UO_indent_func_call_param].b &&
-             (pc->type == CT_FPAREN_OPEN) &&
-             (pc->parent_type == CT_FUNC_CALL))
+         if ((pc->type == CT_FPAREN_OPEN) &&
+             ((cpd.settings[UO_indent_func_call_param].b &&
+               (pc->parent_type == CT_FUNC_CALL))
+              ||
+              (cpd.settings[UO_indent_func_proto_param].b &&
+               ((pc->parent_type == CT_FUNC_PROTO) ||
+                (pc->parent_type == CT_FUNC_CLASS)))
+              ||
+              (cpd.settings[UO_indent_func_class_param].b &&
+               (pc->parent_type == CT_FUNC_CLASS))
+              ||
+              (cpd.settings[UO_indent_func_ctor_var_param].b &&
+               (pc->parent_type == CT_FUNC_CTOR_VAR))
+              ||
+              (cpd.settings[UO_indent_func_def_param].b &&
+               (pc->parent_type == CT_FUNC_DEF))))
          {
-            frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent + indent_size;
+            /* Skip any continuation indents */
+            idx = frm.pse_tos - 1;
+            while ((idx > 0) &&
+                   (frm.pse[idx].type != CT_BRACE_OPEN) &&
+                   (frm.pse[idx].type != CT_VBRACE_OPEN) &&
+                   (frm.pse[idx].type != CT_PAREN_OPEN) &&
+                   (frm.pse[idx].type != CT_FPAREN_OPEN) &&
+                   (frm.pse[idx].type != CT_SPAREN_OPEN) &&
+                   (frm.pse[idx].type != CT_SQUARE_OPEN) &&
+                   (frm.pse[idx].type != CT_ANGLE_OPEN) &&
+                   (frm.pse[idx].type != CT_CLASS_COLON) &&
+                   (frm.pse[idx].type != CT_ASSIGN_NL))
+            {
+               idx--;
+            }
+            frm.pse[frm.pse_tos].indent = frm.pse[idx].indent + indent_size;
+            if (cpd.settings[UO_indent_func_param_double].b)
+            {
+               frm.pse[frm.pse_tos].indent += indent_size;
+            }
+            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
          }
 
          else if ((chunk_is_str(pc, "(", 1) && !cpd.settings[UO_indent_paren_nl].b) ||
@@ -881,7 +1018,9 @@ void indent_text(void)
          frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
          frm.paren_count++;
       }
-      else if ((pc->type == CT_ASSIGN) || (pc->type == CT_IMPORT))
+      else if ((pc->type == CT_ASSIGN) ||
+               (pc->type == CT_IMPORT) ||
+               (pc->type == CT_USING))
       {
          /**
           * if there is a newline after the '=', just indent one level,
@@ -895,6 +1034,10 @@ void indent_text(void)
             if (chunk_is_newline(next))
             {
                frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
+               if (pc->type == CT_ASSIGN)
+               {
+                  frm.pse[frm.pse_tos].type = CT_ASSIGN_NL;
+               }
             }
             else
             {
@@ -921,6 +1064,18 @@ void indent_text(void)
             cout_level = pc->level;
          }
       }
+      else if (pc->type == CT_THROW)
+      {
+         if (pc->parent_type == CT_FUNC_PROTO)
+         {
+            indent_pse_push(frm, pc);
+            frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
+            frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
+            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
+
+            indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
+         }
+      }
       else
       {
          /* anything else? */
@@ -953,7 +1108,7 @@ void indent_text(void)
                   cpd.settings[UO_indent_preserve_sql].b)
          {
             reindent_line(pc, sql_col + (pc->orig_col - sql_orig_col));
-            LOG_FMT(LSYS, "Indent SQL: [%.*s] to %d (%d/%d)\n",
+            LOG_FMT(LINDENT, "Indent SQL: [%.*s] to %d (%d/%d)\n",
                     pc->len, pc->str, pc->column, sql_col, sql_orig_col);
          }
          else if ((pc->type == CT_MEMBER) ||
@@ -1013,17 +1168,17 @@ void indent_text(void)
                chunk_t *ck2 = chunk_get_prev(ck1);
 
                /* If the open paren was the first thing on the line or we are
-                * doing mode 1, then put the close paren in the same column */
+               * doing mode 1, then put the close paren in the same column */
                if (chunk_is_newline(ck2) ||
                    (cpd.settings[UO_indent_paren_close].n == 1))
                {
-                  indent_column = ck1->column;
+                  indent_column_set(ck1->column);
                }
                else
                {
                   if (cpd.settings[UO_indent_paren_close].n != 2)
                   {
-                     indent_column = frm.pse[frm.pse_tos + 1].indent_tmp;
+                     indent_column_set(frm.pse[frm.pse_tos + 1].indent_tmp);
                      if (cpd.settings[UO_indent_paren_close].n == 1)
                      {
                         indent_column--;
@@ -1040,7 +1195,7 @@ void indent_text(void)
             if (cpd.settings[UO_indent_comma_paren].b &&
                 chunk_is_paren_open(frm.pse[frm.pse_tos].pc))
             {
-               indent_column = frm.pse[frm.pse_tos].pc->column;
+               indent_column_set(frm.pse[frm.pse_tos].pc->column);
             }
             LOG_FMT(LINDENT, "%s: %d] comma => %d [%.*s]\n",
                     __func__, pc->orig_line, indent_column, pc->len, pc->str);
@@ -1051,7 +1206,7 @@ void indent_text(void)
             if (cpd.settings[UO_indent_bool_paren].b &&
                 chunk_is_paren_open(frm.pse[frm.pse_tos].pc))
             {
-               indent_column = frm.pse[frm.pse_tos].pc->column;
+               indent_column_set(frm.pse[frm.pse_tos].pc->column);
             }
             LOG_FMT(LINDENT, "%s: %d] bool => %d [%.*s]\n",
                     __func__, pc->orig_line, indent_column, pc->len, pc->str);
@@ -1152,6 +1307,8 @@ void indent_text(void)
               get_token_name(frm.pse[idx].type));
       cpd.error_count++;
    }
+
+   quick_align_again();
 }
 
 /**
