@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: newlines.cpp 991 2007-11-05 14:53:21Z bengardner $
+ * $Id: newlines.cpp 1171 2008-05-02 01:57:37Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -14,11 +14,12 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 
 
 static void newlines_double_space_struct_enum_union(chunk_t *open_brace);
 static bool one_liner_nl_ok(chunk_t *pc);
+static void nl_handle_define(chunk_t *pc);
 
 
 /*
@@ -34,6 +35,29 @@ static bool one_liner_nl_ok(chunk_t *pc);
 
 //#define DEBUG_NEWLINES
 
+static void setup_newline_add(const chunk_t *prev, chunk_t *nl, const chunk_t *next)
+{
+   memset(nl, 0, sizeof(*nl));
+   nl->nl_count = 1;
+   nl->flags    = (prev->flags & PCF_COPY_FLAGS) & ~PCF_IN_PREPROC;
+   if ((prev->flags & PCF_IN_PREPROC) && (next->flags & PCF_IN_PREPROC))
+   {
+      nl->flags |= PCF_IN_PREPROC;
+   }
+   if ((nl->flags & PCF_IN_PREPROC) != 0)
+   {
+      nl->type = CT_NL_CONT;
+      nl->str  = "\\\n";
+      nl->len  = 2;
+   }
+   else
+   {
+      nl->type = CT_NEWLINE;
+      nl->str  = "\n";
+      nl->len  = 1;
+   }
+}
+
 /**
  * 2 parts:
  *  - if/switch/while/for after braces
@@ -42,60 +66,42 @@ static bool one_liner_nl_ok(chunk_t *pc);
  *  - do/while before braces
  */
 
-chunk_t *newline_add_before(chunk_t *pc)
+chunk_t *newline_add_before2(chunk_t *pc, const char *fcn, int line)
 {
    chunk_t nl;
+   chunk_t *prev;
 
-   //fprintf(stderr, "%s: %s line %d\n", __func__, pc->str, pc->orig_line);
+   prev = chunk_get_prev(pc);
+   if (chunk_is_newline(prev))
+   {
+      /* Already has a newline before this chunk */
+      return(prev);
+   }
 
-   memset(&nl, 0, sizeof(nl));
-   nl.nl_count = 1;
-   nl.flags    = pc->flags & PCF_COPY_FLAGS;
-   if ((pc->flags & PCF_IN_PREPROC) != 0)
-   {
-      nl.type = CT_NL_CONT;
-      nl.str  = "\\\n";
-      nl.len  = 2;
-   }
-   else
-   {
-      nl.type = CT_NEWLINE;
-      nl.str  = "\n";
-      nl.len  = 1;
-   }
+   LOG_FMT(LNEWLINE, "%s: '%.*s' on line %d (called from %s line %d)\n",
+           __func__, pc->len, pc->str, pc->orig_line, fcn, line);
+
+   setup_newline_add(prev, &nl, pc);
 
    return(chunk_add_before(&nl, pc));
 }
 
-chunk_t *newline_add_after(chunk_t *pc)
+chunk_t *newline_add_after2(chunk_t *pc, const char *fcn, int line)
 {
    chunk_t nl;
    chunk_t *next;
 
    next = chunk_get_next(pc);
-   if ((next != NULL) && (next->type == CT_NEWLINE))
+   if (chunk_is_newline(next))
    {
       /* Already has a newline after this chunk */
       return(next);
    }
 
-   //fprintf(stderr, "%s: %s line %d\n", __func__, pc->str, pc->orig_line);
+   LOG_FMT(LNEWLINE, "%s: '%.*s' on line %d (called from %s line %d)\n",
+           __func__, pc->len, pc->str, pc->orig_line, fcn, line);
 
-   memset(&nl, 0, sizeof(nl));
-   nl.nl_count = 1;
-   nl.flags    = pc->flags & PCF_COPY_FLAGS;
-   if ((pc->flags & PCF_IN_PREPROC) != 0)
-   {
-      nl.type = CT_NL_CONT;
-      nl.str  = "\\\n";
-      nl.len  = 2;
-   }
-   else
-   {
-      nl.type = CT_NEWLINE;
-      nl.str  = "\n";
-      nl.len  = 1;
-   }
+   setup_newline_add(pc, &nl, next);
 
    return(chunk_add_after(&nl, pc));
 }
@@ -222,13 +228,12 @@ static chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
 #define newline_del_between(start, end) \
    newline_del_between2(start, end, __func__, __LINE__)
 
-static bool newline_del_between2(chunk_t *start, chunk_t *end,
+static void newline_del_between2(chunk_t *start, chunk_t *end,
                                  const char *func, int line)
 {
    chunk_t *next;
    chunk_t *prev;
    chunk_t *pc    = start;
-   bool    retval = false;
 
    LOG_FMT(LNEWLINE, "%s: '%.*s' line %d:%d and '%.*s' line %d:%d : caller=%s:%d preproc=%d/%d\n",
            __func__, start->len, start->str, start->orig_line, start->orig_col,
@@ -239,7 +244,7 @@ static bool newline_del_between2(chunk_t *start, chunk_t *end,
    /* Can't remove anything if the preproc status differs */
    if (!chunk_same_preproc(start, end))
    {
-      return(false);
+      return;
    }
 
    do
@@ -255,8 +260,8 @@ static bool newline_del_between2(chunk_t *start, chunk_t *end,
             if (chunk_safe_to_del_nl(pc))
             {
                chunk_del(pc);
+               align_to_column(next, prev->column + space_col_align(prev, next));
             }
-            retval = true;
          }
          else
          {
@@ -277,10 +282,8 @@ static bool newline_del_between2(chunk_t *start, chunk_t *end,
       if (chunk_get_prev_nl(end) != start)
       {
          chunk_move_after(end, start);
-         retval = true;
       }
    }
-   return(retval);
 }
 
 /**
@@ -352,6 +355,7 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
 {
    chunk_t *pc;
    chunk_t *prev;
+   chunk_t *next;
    chunk_t *last_nl = NULL;
    int     level    = start->level;
    bool    do_add   = nl_opt & AV_ADD;
@@ -386,10 +390,11 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
                while (chunk_is_newline(prev = chunk_get_prev_nvb(pc)))
                {
                   /* Make sure we don't combine a preproc and non-preproc */
-                  if (chunk_safe_to_del_nl(prev))
+                  if (!chunk_safe_to_del_nl(prev))
                   {
-                     chunk_del(prev);
+                     break;
                   }
+                  chunk_del(prev);
                }
             }
 
@@ -408,20 +413,28 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
       }
       else
       {
-         if (do_add)/* we found something previously besides a comment or a new line */
+         if (do_add) /* we found something previously besides a comment or a new line */
          {
             /* if we have run across a newline */
-            if (last_nl)
+            if (last_nl != NULL)
             {
                if (last_nl->nl_count < 2)
                {
                   last_nl->nl_count = 2;
                }
             }
-            /* if we didn't run into a nl, need to add one */
-            else if ((last_nl = newline_add_after(pc)) != NULL)
+            else
             {
-               last_nl->nl_count = 2;
+               /* we didn't run into a nl, so we need to add one */
+               if (((next = chunk_get_next(pc)) != NULL) &&
+                   chunk_is_comment(next))
+               {
+                  pc = next;
+               }
+               if ((last_nl = newline_add_after(pc)) != NULL)
+               {
+                  last_nl->nl_count = 2;
+               }
             }
          }
 
@@ -461,12 +474,9 @@ static void remove_next_newlines(chunk_t *start)
 
    while ((next = chunk_get_next(start)) != NULL)
    {
-      if (chunk_is_newline(next))
+      if (chunk_is_newline(next) && chunk_safe_to_del_nl(next))
       {
-         if (chunk_safe_to_del_nl(next))
-         {
-            chunk_del(next);
-         }
+         chunk_del(next);
       }
       else if (chunk_is_vbrace(next))
       {
@@ -554,7 +564,7 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
          prev->nl_count = 1;
          remove_next_newlines(pc);
       }
-      else if (chunk_is_newline(next = chunk_get_next_nvb(pc)))/* otherwise just deal with nls after brace */
+      else if (chunk_is_newline(next = chunk_get_next_nvb(pc))) /* otherwise just deal with nls after brace */
       {
          next->nl_count = 1;
          remove_next_newlines(next);
@@ -582,6 +592,13 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
          /* if we have no newlines, add one and make it double */
          if (nl_count == 0)
          {
+
+            if (((next = chunk_get_next(pc)) != NULL) &&
+                chunk_is_comment(next))
+            {
+               pc = next;
+            }
+
             if ((next = newline_add_after(pc)) == NULL)
             {
                return;
@@ -589,7 +606,7 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
 
             next->nl_count = 2;
          }
-         else if (nl_count == 1)/* if we don't have enough newlines */
+         else if (nl_count == 1) /* if we don't have enough newlines */
          {
             /* if we have one before vbrace, need to add one after */
             if (have_pre_vbrace_nl)
@@ -1106,7 +1123,11 @@ static void newline_func_def(chunk_t *start)
    }
 
    /* Handle break newlines type and function */
-   if (cpd.settings[UO_nl_func_type_name].a != AV_IGNORE)
+   argval_t a = (start->parent_type == CT_FUNC_PROTO) ?
+                cpd.settings[UO_nl_func_proto_type_name].a :
+                cpd.settings[UO_nl_func_type_name].a;
+
+   if (a != AV_IGNORE)
    {
       prev = chunk_get_prev_ncnl(start);
       prev = chunk_get_prev_ncnl(prev);
@@ -1122,7 +1143,7 @@ static void newline_func_def(chunk_t *start)
 
       if (prev != NULL)
       {
-         newline_iarf(prev, cpd.settings[UO_nl_func_type_name].a);
+         newline_iarf(prev, a);
       }
    }
 
@@ -1162,7 +1183,6 @@ static void newline_func_def(chunk_t *start)
    }
 }
 
-
 /**
  * Checks to see if it is OK to add a newline around the chunk.
  * Don't want to break one-liners...
@@ -1247,7 +1267,6 @@ static void nl_create_one_liner(chunk_t *vbrace_open)
    }
 }
 
-
 /**
  * Step through all chunks.
  */
@@ -1474,6 +1493,10 @@ void newlines_cleanup_braces(void)
             newline_case(pc);
          }
       }
+      else if (pc->type == CT_THROW)
+      {
+         newline_iarf(chunk_get_prev_ncnl(pc), cpd.settings[UO_nl_before_throw].a);
+      }
       else if (pc->type == CT_CASE_COLON)
       {
          if (cpd.settings[UO_nl_after_case].b)
@@ -1510,7 +1533,8 @@ void newlines_cleanup_braces(void)
             {
                next = chunk_get_next(next);
             }
-            if (!chunk_is_comment(next) &&
+            if ((next != NULL) &&
+                !chunk_is_comment(next) &&
                 !chunk_is_newline(next))
             {
                if (one_liner_nl_ok(next))
@@ -1529,6 +1553,7 @@ void newlines_cleanup_braces(void)
               (cpd.settings[UO_nl_func_decl_args].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_decl_end].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_type_name].a != AV_IGNORE) ||
+              (cpd.settings[UO_nl_func_proto_type_name].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_paren].a != AV_IGNORE)))
          {
             newline_func_def(pc);
@@ -1603,6 +1628,13 @@ void newlines_cleanup_braces(void)
             }
          }
       }
+      else if (pc->type == CT_PP_DEFINE)
+      {
+         if (cpd.settings[UO_nl_multi_line_define].b)
+         {
+            nl_handle_define(pc);
+         }
+      }
       else
       {
          /* ignore it */
@@ -1610,6 +1642,35 @@ void newlines_cleanup_braces(void)
    }
 }
 
+/**
+ * Find the next newline or nl_cont
+ */
+static void nl_handle_define(chunk_t *pc)
+{
+   chunk_t *nl  = pc;
+   chunk_t *ref = NULL;
+
+   while ((nl = chunk_get_next(nl)) != NULL)
+   {
+      if (nl->type == CT_NEWLINE)
+      {
+         return;
+      }
+      if ((nl->type == CT_MACRO) ||
+          ((nl->type == CT_FPAREN_CLOSE) && (nl->parent_type == CT_MACRO_FUNC)))
+      {
+         ref = nl;
+      }
+      if (nl->type == CT_NL_CONT)
+      {
+         if (ref != NULL)
+         {
+            newline_add_after(ref);
+         }
+         return;
+      }
+   }
+}
 
 void newline_after_multiline_comment(void)
 {
@@ -1907,14 +1968,16 @@ void newlines_class_colon_pos(void)
 
          if (mode == TP_TRAIL)
          {
-            if (chunk_is_newline(prev) && (prev->nl_count == 1))
+            if (chunk_is_newline(prev) && (prev->nl_count == 1) &&
+                chunk_safe_to_del_nl(prev))
             {
                chunk_swap(pc, prev);
             }
          }
          else if (mode == TP_LEAD)
          {
-            if (chunk_is_newline(next) && (next->nl_count == 1))
+            if (chunk_is_newline(next) && (next->nl_count == 1) &&
+                chunk_safe_to_del_nl(next))
             {
                chunk_swap(pc, next);
             }
@@ -1936,7 +1999,7 @@ void newlines_class_colon_pos(void)
                {
                   newline_add_after(pc);
                }
-               else
+               else if (cpd.settings[UO_pos_class_comma].tp == TP_LEAD)
                {
                   newline_add_before(pc);
                   next = chunk_get_next_nc(pc);
@@ -2130,7 +2193,6 @@ void newlines_cleanup_dup(void)
    }
 }
 
-
 /**
  * Make sure there is a blank line after a commented group of values
  */
