@@ -5,19 +5,20 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: options.cpp 997 2007-11-08 00:06:09Z bengardner $
+ * $Id: options.cpp 1192 2008-06-06 19:12:54Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "args.h"
 #include "prototypes.h"
+#include "uncrustify_version.h"
 #include <cstring>
 #ifdef HAVE_STRINGS_H
-#include <strings.h> /* strcasecmp() */
+ #include <strings.h> /* strcasecmp() */
 #endif
 #include <cstdio>
 #include <cstdlib>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 
 
 static std::map<std::string, option_map_value>      option_name_map;
@@ -134,6 +135,8 @@ void register_options(void)
                   "Add or remove space around arithmetic operator '+', '-', '/', '*', etc");
    unc_add_option("sp_assign", UO_sp_assign, AT_IARF,
                   "Add or remove space around assignment operator '=', '+=', etc");
+   unc_add_option("sp_enum_assign", UO_sp_enum_assign, AT_IARF,
+                  "Add or remove space around assignment '=' in enum");
    unc_add_option("sp_bool", UO_sp_bool, AT_IARF,
                   "Add or remove space around boolean operators '&&' and '||'");
    unc_add_option("sp_compare", UO_sp_compare, AT_IARF,
@@ -148,14 +151,22 @@ void register_options(void)
                   "Add or remove space between ')' and '{'");
    unc_add_option("sp_before_ptr_star", UO_sp_before_ptr_star, AT_IARF,
                   "Add or remove space before pointer star '*'");
+   unc_add_option("sp_before_unnamed_ptr_star", UO_sp_before_unnamed_ptr_star, AT_IARF,
+                  "Add or remove space before pointer star '*' that isn't followed by a variable name\n"
+                  "If set to 'ignore', sp_before_ptr_star is used instead.");
    unc_add_option("sp_between_ptr_star", UO_sp_between_ptr_star, AT_IARF,
                   "Add or remove space between pointer stars '*'");
    unc_add_option("sp_after_ptr_star", UO_sp_after_ptr_star, AT_IARF,
                   "Add or remove space after pointer star '*', if followed by a word.");
    unc_add_option("sp_before_byref", UO_sp_before_byref, AT_IARF,
                   "Add or remove space before reference sign '&'");
+   unc_add_option("sp_before_unnamed_byref", UO_sp_before_unnamed_byref, AT_IARF,
+                  "Add or remove space before reference sign '&' that isn't followed by a variable name\n"
+                  "If set to 'ignore', sp_before_byref is used instead.");
    unc_add_option("sp_after_byref", UO_sp_after_byref, AT_IARF,
                   "Add or remove space after reference sign '&', if followed by a word.");
+   unc_add_option("sp_after_type", UO_sp_after_type, AT_IARF,
+                  "Add or remove space between type and word");
    unc_add_option("sp_before_angle", UO_sp_before_angle, AT_IARF,
                   "Add or remove space before '<>'");
    unc_add_option("sp_after_angle", UO_sp_after_angle, AT_IARF,
@@ -192,6 +203,10 @@ void register_options(void)
                   "Add or remove space after ','");
    unc_add_option("sp_before_comma", UO_sp_before_comma, AT_IARF,
                   "Add or remove space before ','");
+   unc_add_option("sp_after_class_colon", UO_sp_after_class_colon, AT_IARF,
+                  "Add or remove space after class ':'");
+   unc_add_option("sp_before_class_colon", UO_sp_before_class_colon, AT_IARF,
+                  "Add or remove space before class ':'");
    unc_add_option("sp_after_operator", UO_sp_after_operator, AT_IARF,
                   "Add or remove space between 'operator' and operator sign");
    unc_add_option("sp_after_cast", UO_sp_after_cast, AT_IARF,
@@ -208,6 +223,8 @@ void register_options(void)
                   "Add or remove space inside struct/union '{' and '}'");
    unc_add_option("sp_inside_braces", UO_sp_inside_braces, AT_IARF,
                   "Add or remove space inside '{' and '}'");
+   unc_add_option("sp_inside_braces_empty", UO_sp_inside_braces_empty, AT_IARF,
+                  "Add or remove space inside '{}'");
    unc_add_option("sp_inside_angle", UO_sp_inside_angle, AT_IARF,
                   "Add or remove space inside '<' and '>'");
    unc_add_option("sp_type_func", UO_sp_type_func, AT_IARF,
@@ -265,9 +282,24 @@ void register_options(void)
    unc_add_option("sp_addr", UO_sp_addr, AT_IARF,
                   "Add or remove space after the '&' (address-of) operator.\n"
                   "This does not affect the spacing after a '&' that is part of a type.");
+   unc_add_option("sp_member", UO_sp_member, AT_IARF,
+                  "Add or remove space around the '.' or '->' operators\n");
    unc_add_option("sp_deref", UO_sp_deref, AT_IARF,
                   "Add or remove space after the '*' (dereference) operator.\n"
                   "This does not affect the spacing after a '*' that is part of a type.");
+   unc_add_option("sp_sign", UO_sp_sign, AT_IARF,
+                  "Add or remove space after '+' or '-', as in 'x = -5' or 'y = +7'");
+   unc_add_option("sp_incdec", UO_sp_incdec, AT_IARF,
+                  "Add or remove space before or after '++' and '--', as in '(--x)' or 'y++;'");
+
+   unc_add_option("sp_after_oc_scope", UO_sp_after_oc_scope, AT_IARF,
+                  "Add or remove space after the scope '+' or '-', as in '-(void) foo;' or '+(int) bar;'");
+   unc_add_option("sp_before_oc_colon", UO_sp_before_oc_colon, AT_IARF,
+                  "Add or remove space after the colon in message specs\n"
+                  "'-(int) f: (int) x;' vs '+(int) f : (int) x;'");
+   unc_add_option("sp_after_oc_type", UO_sp_after_oc_type, AT_IARF,
+                  "Add or remove space after the (type) in message specs\n"
+                  "'-(int) f: (int) x;' vs '+(int)f : (int)x;'");
 
    unc_begin_group(UG_indent, "Indenting");
    unc_add_option("indent_columns", UO_indent_columns, AT_NUM,
@@ -293,17 +325,35 @@ void register_options(void)
                   "Indent based on the size of the brace parent, ie 'if' => 3 spaces, 'for' => 4 spaces, etc.");
    unc_add_option("indent_namespace", UO_indent_namespace, AT_BOOL,
                   "Whether the 'namespace' body is indented");
+   unc_add_option("indent_extern", UO_indent_extern, AT_BOOL,
+                  "Whether the 'extern \"C\"' body is indented");
    unc_add_option("indent_class", UO_indent_class, AT_BOOL,
                   "Whether the 'class' body is indented");
    unc_add_option("indent_class_colon", UO_indent_class_colon, AT_BOOL,
                   "Whether to indent the stuff after a leading class colon");
+
    unc_add_option("indent_func_call_param", UO_indent_func_call_param, AT_BOOL,
-                  "Whether to indent continued function call parameters one indent level (true) or aligns instead of indent (false)");
+                  "True:  indent continued function call parameters one indent level\n"
+                  "False: align parameters under the open paren");
+   unc_add_option("indent_func_def_param", UO_indent_func_def_param, AT_BOOL,
+                  "Same as indent_func_call_param, but for function defs");
+   unc_add_option("indent_func_proto_param", UO_indent_func_proto_param, AT_BOOL,
+                  "Same as indent_func_call_param, but for function protos");
+   unc_add_option("indent_func_class_param", UO_indent_func_class_param, AT_BOOL,
+                  "Same as indent_func_call_param, but for class declarations");
+   unc_add_option("indent_func_ctor_var_param", UO_indent_func_ctor_var_param, AT_BOOL,
+                  "Same as indent_func_call_param, but for class variable constructors");
+   unc_add_option("indent_func_param_double", UO_indent_func_param_double, AT_BOOL,
+                  "Double the indent for indent_func_xxx_param options");
+
    unc_add_option("indent_member", UO_indent_member, AT_NUM,
                   "The number of spaces to indent a continued '->' or '.'\n"
                   "Usually set to 0, 1, or indent_columns.");
    unc_add_option("indent_sing_line_comments", UO_indent_sing_line_comments, AT_NUM,
                   "Spaces to indent single line ('//') comments on lines before code");
+   unc_add_option("indent_relative_single_line_comments", UO_indent_relative_single_line_comments, AT_BOOL,
+                  "If set, will indent trailing single line ('//') comments relative\n"
+                  "to the code instead of trying to keep the same absolute column");
    unc_add_option("indent_switch_case", UO_indent_switch_case, AT_NUM,
                   "Spaces to indent 'case' from 'switch'\n"
                   "Usually 0 or indent_columns.");
@@ -322,6 +372,9 @@ void register_options(void)
                   " <=0 : subtract from brace indent", "", -16, 16);
    unc_add_option("indent_access_spec", UO_indent_access_spec, AT_NUM,
                   "Same as indent_label, but for access specifiers that are followed by a colon", "", -16, 16);
+   unc_add_option("indent_access_spec_body", UO_indent_access_spec_body, AT_BOOL,
+                  "Indent the code after an access specifier by one level.\n"
+                  "If set, this option forces 'indent_access_spec=0'");
    unc_add_option("indent_paren_nl", UO_indent_paren_nl, AT_BOOL,
                   "If an open paren is followed by a newline, indent the next line so that it lines up after the open paren (not recommended)");
    unc_add_option("indent_paren_close", UO_indent_paren_close, AT_NUM,
@@ -415,8 +468,12 @@ void register_options(void)
                   "Add or remove newline between 'switch' and '{'");
    unc_add_option("nl_multi_line_cond", UO_nl_multi_line_cond, AT_BOOL,
                   "Add or remove newline when condition spans two or more lines");
+   unc_add_option("nl_multi_line_define", UO_nl_multi_line_define, AT_BOOL,
+                  "Force a newline in a define after the macro name for multi-line defines.");
    unc_add_option("nl_before_case", UO_nl_before_case, AT_BOOL,
                   "Whether to put a newline before 'case' statement");
+   unc_add_option("nl_before_throw", UO_nl_before_throw, AT_IARF,
+                  "Add or remove newline between ')' and 'throw'");
    unc_add_option("nl_after_case", UO_nl_after_case, AT_BOOL,
                   "Whether to put a newline after 'case' statement");
    unc_add_option("nl_namespace_brace", UO_nl_namespace_brace, AT_IARF,
@@ -429,6 +486,8 @@ void register_options(void)
                   "Add or remove newline after each ',' in the constructor member initialization");
    unc_add_option("nl_func_type_name", UO_nl_func_type_name, AT_IARF,
                   "Add or remove newline between return type and function name in definition");
+   unc_add_option("nl_func_proto_type_name", UO_nl_func_proto_type_name, AT_IARF,
+                  "Add or remove newline between return type and function name in a prototype");
    unc_add_option("nl_func_paren", UO_nl_func_paren, AT_IARF,
                   "Add or remove newline between a function name and the opening '('");
    unc_add_option("nl_func_decl_start", UO_nl_func_decl_start, AT_IARF,
@@ -572,6 +631,8 @@ void register_options(void)
                   " 2=Dangling", "", 0, 2);
    unc_add_option("align_var_def_thresh", UO_align_var_def_thresh, AT_NUM,
                   "The threshold for aligning variable definitions (0=no limit)", "", 0, 5000);
+   unc_add_option("align_var_def_gap", UO_align_var_def_gap, AT_NUM,
+                  "The gap for aligning variable definitions");
    unc_add_option("align_var_def_colon", UO_align_var_def_colon, AT_BOOL,
                   "Whether to align the colon in struct bit fields");
    unc_add_option("align_var_def_inline", UO_align_var_def_inline, AT_BOOL,
@@ -588,6 +649,8 @@ void register_options(void)
                   "The span for aligning struct/union (0=don't align)", "", 0, 5000);
    unc_add_option("align_var_struct_thresh", UO_align_var_struct_thresh, AT_NUM,
                   "The threshold for aligning struct/union member definitions (0=no limit)", "", 0, 5000);
+   unc_add_option("align_var_struct_gap", UO_align_var_struct_gap, AT_NUM,
+                  "The gap for aligning struct/union member definitions");
    unc_add_option("align_struct_init_span", UO_align_struct_init_span, AT_NUM,
                   "The span for aligning struct initializer values (0=don't align)", "", 0, 5000);
    unc_add_option("align_typedef_gap", UO_align_typedef_gap, AT_NUM,
@@ -611,10 +674,27 @@ void register_options(void)
                   "2: The '&' is part of the type, but dangling: typedef int &pint;", "", 0, 2);
    unc_add_option("align_right_cmt_span", UO_align_right_cmt_span, AT_NUM,
                   "The span for aligning comments that end lines (0=don't align)", "", 0, 5000);
+   unc_add_option("align_right_cmt_mix", UO_align_right_cmt_mix, AT_BOOL,
+                  "If aligning comments, mix with comments after '}' and #endif with less than 3 spaces before the comment");
    unc_add_option("align_func_proto_span", UO_align_func_proto_span, AT_NUM,
                   "The span for aligning function prototypes (0=don't align)", "", 0, 5000);
+   unc_add_option("align_func_proto_gap", UO_align_func_proto_gap, AT_NUM,
+                  "Minimum gap between the return type and the function name.");
+   unc_add_option("align_mix_var_proto", UO_align_mix_var_proto, AT_BOOL,
+                  "Whether to mix aligning prototype and variable declarations.\n"
+                  "If true, align_var_def_XXX options are used instead of align_func_proto_XXX options.");
+   unc_add_option("align_single_line_func", UO_align_single_line_func, AT_BOOL,
+                  "Align single-line functions with function prototypes, uses align_func_proto_span");
+   unc_add_option("align_single_line_brace", UO_align_single_line_brace, AT_BOOL,
+                  "Aligning the open brace of single-line functions.\n"
+                  "Requires align_single_line_func=true, uses align_func_proto_span");
+   unc_add_option("align_single_line_brace_gap", UO_align_single_line_brace_gap, AT_NUM,
+                  "Gap for align_single_line_brace.\n");
+   unc_add_option("align_oc_msg_spec_span", UO_align_oc_msg_spec_span, AT_NUM,
+                  "The span for aligning ObjC msg spec (0=don't align)", "", 0, 5000);
    unc_add_option("align_nl_cont", UO_align_nl_cont, AT_BOOL,
-                  "Whether to align macros wrapped with a backslash and a newline");
+                  "Whether to align macros wrapped with a backslash and a newline.\n"
+                  "This will not work right if the macro contains a multi-line comment.");
    unc_add_option("align_pp_define_gap", UO_align_pp_define_gap, AT_NUM,
                   "The minimum space between label and value of a preprocessor define");
    unc_add_option("align_pp_define_span", UO_align_pp_define_span, AT_NUM,
@@ -622,7 +702,9 @@ void register_options(void)
 
    unc_begin_group(UG_comment, "Comment modifications");
    unc_add_option("cmt_width", UO_cmt_width, AT_NUM,
-                  "Try to wrap comments at cmt_witdth columns", "", 16, 256);
+                  "Try to wrap comments at cmt_width columns", "", 16, 256);
+   unc_add_option("cmt_indent_multi", UO_cmt_indent_multi, AT_BOOL,
+                  "Whether to mess with the indent of multi-line comments");
    unc_add_option("cmt_c_group", UO_cmt_c_group, AT_BOOL,
                   "Whether to group c-comments that look like they are in a block");
    unc_add_option("cmt_c_nl_start", UO_cmt_c_nl_start, AT_BOOL,
@@ -676,6 +758,21 @@ void register_options(void)
                   "Add parens on 'while' and 'if' statement around bools");
    unc_add_option("mod_remove_extra_semicolon", UO_mod_remove_extra_semicolon, AT_BOOL,
                   "Whether to remove superfluous semicolons");
+   unc_add_option("mod_add_long_function_closebrace_comment", UO_mod_add_long_function_closebrace_comment, AT_NUM,
+                  "If a function body exceeds the specified number of newlines and doesn't have a comment after\n"
+                  "the close brace, a comment will be added.");
+   unc_add_option("mod_add_long_switch_closebrace_comment", UO_mod_add_long_switch_closebrace_comment, AT_NUM,
+                  "If a switch body exceeds the specified number of newlines and doesn't have a comment after\n"
+                  "the close brace, a comment will be added.");
+   unc_add_option("mod_sort_import", UO_mod_sort_import, AT_BOOL,
+                  "If TRUE, will sort consecutive single-line 'import' statements [Java, D]");
+   unc_add_option("mod_sort_using", UO_mod_sort_using, AT_BOOL,
+                  "If TRUE, will sort consecutive single-line 'using' statements [C#]");
+   unc_add_option("mod_sort_include", UO_mod_sort_include, AT_BOOL,
+                  "If TRUE, will sort consecutive single-line '#include' statements [C/C++] and '#import' statements [Obj-C]\n"
+                  "This is generally a bad idea, as it may break your code.");
+   unc_add_option("mod_move_case_break", UO_mod_move_case_break, AT_BOOL,
+                  "If TRUE, it will move a 'break' that appears after a fully braced 'case' before the close brace.");
 
    unc_begin_group(UG_preprocessor, "Preprocessor options");
    unc_add_option("pp_indent", UO_pp_indent, AT_IARF,
@@ -729,7 +826,7 @@ const option_map_value *get_option_name(int uo)
 /**
  * Convert the value string to the correct type in dest.
  */
-static void convert_value(const option_map_value *entry, const char *val, op_val_t* dest)
+static void convert_value(const option_map_value *entry, const char *val, op_val_t *dest)
 {
    const option_map_value *tmp;
    bool btrue;
@@ -788,8 +885,8 @@ static void convert_value(const option_map_value *entry, const char *val, op_val
 
    if (entry->type == AT_NUM)
    {
-      if (isdigit(*val) ||
-          (isdigit(val[1]) && ((*val == '-') || (*val == '+'))))
+      if (unc_isdigit(*val) ||
+          (unc_isdigit(val[1]) && ((*val == '-') || (*val == '+'))))
       {
          dest->n = strtol(val, NULL, 0);
          return;
@@ -1012,6 +1109,8 @@ int save_option_file(FILE *pfile, bool withDoc)
    int         name_len;
    int         idx;
 
+   fprintf(pfile, "# Uncrustify %s\n", UNCRUSTIFY_VERSION);
+
    /* Print the options by group */
    for (group_map_it jt = group_map.begin(); jt != group_map.end(); jt++)
    {
@@ -1085,7 +1184,7 @@ int save_option_file(FILE *pfile, bool withDoc)
               "# macro-open  BEGIN_TEMPLATE_MESSAGE_MAP\n"
               "# macro-open  BEGIN_MESSAGE_MAP\n"
               "# macro-close END_MESSAGE_MAP\n"
-      );
+              );
    }
 
    /* Print custom keywords */
@@ -1145,6 +1244,7 @@ void print_options(FILE *pfile, bool verbose)
       "Number",
       "{ Auto, LF, CR, CRLF }",
       "{ Ignore, Lead, Trail }",
+      "String",
    };
 
    option_name_map_it it;
@@ -1160,6 +1260,8 @@ void print_options(FILE *pfile, bool verbose)
    }
    max_width++;
 
+   fprintf(pfile, "# Uncrustify %s\n", UNCRUSTIFY_VERSION);
+
    /* Print the all out */
    for (group_map_it jt = group_map.begin(); jt != group_map.end(); jt++)
    {
@@ -1214,6 +1316,11 @@ void set_option_defaults(void)
    cpd.settings[UO_sp_inv].a             = AV_REMOVE;
    cpd.settings[UO_sp_addr].a            = AV_REMOVE;
    cpd.settings[UO_sp_deref].a           = AV_REMOVE;
+   cpd.settings[UO_sp_member].a          = AV_REMOVE;
+   cpd.settings[UO_sp_sign].a            = AV_REMOVE;
+   cpd.settings[UO_sp_incdec].a          = AV_REMOVE;
+   cpd.settings[UO_sp_after_type].a      = AV_FORCE;
+   cpd.settings[UO_cmt_indent_multi].b   = true;
 }
 
 std::string argtype_to_string(argtype_e argtype)
