@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: options.h 997 2007-11-08 00:06:09Z bengardner $
+ * $Id: options.h 1192 2008-06-06 19:12:54Z bengardner $
  */
 #ifndef OPTIONS_H_INCLUDED
 #define OPTIONS_H_INCLUDED
@@ -122,6 +122,7 @@ enum uncrustify_options
    UO_indent_brace_parent,       // indent the braces based on the parent size (if=3, for=4, etc)
    UO_indent_label,              // 0=left >0=col from left, <0=sub from brace indent
    UO_indent_access_spec,        // same as indent_label, but for "private:", "public:"
+   UO_indent_access_spec_body,   // indent private/public/protected inside a class (overrides indent_access_spec)
 
    UO_indent_align_string,       // True/False - indent align broken strings
    UO_indent_xml_string,         // Number amount to indent XML strings
@@ -129,14 +130,21 @@ enum uncrustify_options
    UO_indent_col1_comment,       // indent comments in column 1
 
    UO_indent_func_call_param,    // indent continued function calls to indent_columns
+   UO_indent_func_proto_param,   // same, but for function protos
+   UO_indent_func_def_param,     // same, but for function defs
+   UO_indent_func_class_param,   // same, but for classes
+   UO_indent_func_ctor_var_param,
+   UO_indent_func_param_double,  // double the tab indent for
 
    UO_indent_namespace,          // indent stuff inside namespace braces
+   UO_indent_extern,
    UO_indent_class,              // indent stuff inside class braces
    UO_indent_class_colon,        // indent stuff after a class colon
 
    UO_indent_member,             // indent lines broken at a member '.' or '->'
 
    UO_indent_sing_line_comments, // indent single line ('//') comments on lines before code
+   UO_indent_relative_single_line_comments, // indent single line ('//') comments after code
    UO_indent_preserve_sql,       // preserve indent of EXEC SQL statement body
 
    /*
@@ -151,8 +159,11 @@ enum uncrustify_options
    UO_sp_inside_paren_cast,     // spaces inside the parens of a cast
 
    UO_sp_before_byref,          // space before '&' of 'fcn(int& idx)'
+   UO_sp_before_unnamed_byref,
    UO_sp_after_byref,           // space after a '&'  as in 'int& var'
 
+   UO_sp_after_type,            // space between type and word
+
    UO_sp_inside_fparen,         // space inside 'foo( xxx )' vs 'foo(xxx)'
    UO_sp_inside_fparens,        // space inside 'foo( )' vs 'foo()'
    UO_sp_inside_paren,          // space inside '+ ( xxx )' vs '+ (xxx)'
@@ -184,6 +195,10 @@ enum uncrustify_options
    UO_sp_bool,                  // space around || &&
    UO_sp_compare,               // space around < > ==, etc
    UO_sp_assign,                // space around =, +=, etc
+   UO_sp_enum_assign,           // space around = in enum
+
+   UO_sp_after_class_colon,     // space after class ':'
+   UO_sp_before_class_colon,    // space before class ':'
 
    UO_sp_func_def_paren,        // space between 'func' and '(' - "foo (" vs "foo("
    UO_sp_func_call_paren,       // space between 'func' and '(' - "foo (" vs "foo("
@@ -195,6 +210,7 @@ enum uncrustify_options
    UO_sp_type_func,             // space between return type and 'func'
    // a minimum of 1 is forced except for '*'
    UO_sp_before_ptr_star,       // space before a '*' that is part of a type
+   UO_sp_before_unnamed_ptr_star,
    UO_sp_after_ptr_star,        // space after a '*' that is part of a type
    UO_sp_between_ptr_star,      // space between two '*' that are part of a type
 
@@ -205,6 +221,7 @@ enum uncrustify_options
    UO_sp_before_semi_for_empty, // space before ';' in empty for statement
    UO_sp_after_semi_for_empty,  // space after final ';' in empty for statement
    UO_sp_inside_braces,         // space inside '{' and '}' - "{ 1, 2, 3 }"
+   UO_sp_inside_braces_empty,   // space inside '{' and '}' - "{ }"
    UO_sp_inside_braces_enum,    // space inside enum '{' and '}' - "{ a, b, c }"
    UO_sp_inside_braces_struct,  // space inside struct/union '{' and '}'
 
@@ -232,6 +249,12 @@ enum uncrustify_options
    UO_sp_inv,
    UO_sp_addr,
    UO_sp_deref,
+   UO_sp_member,
+   UO_sp_sign,
+   UO_sp_incdec,
+   UO_sp_after_oc_scope,
+   UO_sp_before_oc_colon,
+   UO_sp_after_oc_type,
 
    /*
     * Line splitting options (for long lines)
@@ -259,15 +282,19 @@ enum uncrustify_options
    UO_align_assign_span,          // align on '='. 0=don't align
    UO_align_assign_thresh,        // threshold for aligning on '='. 0=no limit
    UO_align_right_cmt_span,       // align comment that end lines. 0=don't align
+   UO_align_right_cmt_mix,        // mix comments after '}' and preproc with others
    UO_align_func_params,          // align prototype variable defs on variable
    UO_align_var_def_span,         // align variable defs on variable (span for regular stuff)
    UO_align_var_def_thresh,       // align variable defs threshold
+   UO_align_var_def_gap,          // align variable defs gap
+   UO_align_var_def_colon_gap,    // align variable defs gap for bit colons
    UO_align_var_def_inline,       // also align inline struct/enum/union var defs
    UO_align_var_def_star_style,   // see UO_align_typedef_star_style
    UO_align_var_def_amp_style,    // see UO_align_typedef_star_style
    UO_align_var_def_colon,        // align the colon in struct bit fields
    UO_align_var_struct_span,      // span for struct/union (0=don't align)
    UO_align_var_struct_thresh,    // threshold for struct/union, 0=no limit
+   UO_align_var_struct_gap,       // gap for struct/union
    UO_align_pp_define_span,       // align bodies in #define statments
    //UO_align_pp_define_col_min,    //TODO: min column for a #define value
    //UO_align_pp_define_col_max,    //TODO: max column for a #define value
@@ -276,6 +303,12 @@ enum uncrustify_options
    //UO_align_enum_col_max,         //TODO: the max column for enum '=' alignment
    UO_align_struct_init_span,     // align structure initializer values
    UO_align_func_proto_span,      // align function prototypes
+   UO_align_func_proto_gap,       // align function prototypes
+   UO_align_mix_var_proto,        // mix function prototypes and variable decl
+   UO_align_single_line_func,     // mix single line function with prototypes
+   UO_align_single_line_brace,    // align the open brace of single line functions
+   UO_align_single_line_brace_gap,// gap for align_single_line_brace
+   UO_align_oc_msg_spec_span,     // align ObjC msg spec
    UO_align_number_left,          // left-align numbers (not fully supported, yet)
    UO_align_typedef_span,         // align single-line typedefs
    UO_align_typedef_gap,          // minimum spacing
@@ -298,8 +331,10 @@ enum uncrustify_options
    UO_nl_func_decl_args,          // newline after each ',' in a function decl
    UO_nl_func_decl_end,           // newline before the ')' in a function decl
    UO_nl_func_type_name,          // newline between return type and func name in def
+   UO_nl_func_proto_type_name,    // nl_func_type_name, but for prottypes
    UO_nl_func_var_def_blk,        // newline after a block of variable defs
    UO_nl_before_case,             // newline before 'case' statement
+   UO_nl_before_throw,
    UO_nl_after_return,            /* newline after return statement */
    UO_nl_after_case,              /* disallow nested "case 1: a=3;" */
    UO_nl_after_semicolon,         // disallow multiple statements on a line "a=1;b=4;"
@@ -335,6 +370,8 @@ enum uncrustify_options
    UO_nl_multi_line_cond,         /* nl between ) and { when cond spans >=2 lines */
    UO_nl_elseif_brace,            // nl between close paren and open brace in 'else if () {'
 
+   UO_nl_multi_line_define,       // nl after define XXX for multi-line define
+
    UO_nl_before_if,               // nl before if
    UO_nl_after_if,                // nl after if/else
    UO_nl_before_for,              // nl before for
@@ -412,6 +449,12 @@ enum uncrustify_options
    UO_mod_full_brace_function,    // add optional braces on Pawn functions
    UO_mod_full_paren_if_bool,
    UO_mod_remove_extra_semicolon, // remove extra semicolons
+   UO_mod_add_long_function_closebrace_comment,
+   UO_mod_add_long_switch_closebrace_comment,
+   UO_mod_sort_import,
+   UO_mod_sort_using,
+   UO_mod_sort_include,
+   UO_mod_move_case_break,
 
 
    /*
@@ -419,6 +462,7 @@ enum uncrustify_options
     */
 
    UO_cmt_width,                // column to wrap comments
+   UO_cmt_indent_multi,         // change left indent of multiline comments
    UO_cmt_star_cont,            // put a star on subsequent comment lines
    UO_cmt_sp_before_star_cont,  // # of spaces for subsequent comment lines (before possible star)
    UO_cmt_sp_after_star_cont,   // # of spaces for subsequent comment lines (after star)
