@@ -5,12 +5,13 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: output.cpp 985 2007-10-31 01:12:15Z bengardner $
+ * $Id: output.cpp 1191 2008-06-06 17:55:28Z bengardner $
  */
 
 #include "uncrustify_types.h"
 #include "prototypes.h"
 #include "chunk_list.h"
+#include "unc_ctype.h"
 #include <cstring>
 #include <cstdlib>
 
@@ -34,6 +35,8 @@ static void add_comment_text(const char *text, int len,
 
 void add_char(char ch)
 {
+   static char last_char = 0;
+
    /* convert a newline into the LF/CRLF/CR sequence */
    if (ch == '\n')
    {
@@ -43,16 +46,30 @@ void add_char(char ch)
    }
    else
    {
-      fputc(ch, cpd.fout);
-      if (ch == '\t')
+      /* Explicitly disallow a tab after a space */
+      if ((ch == '\t') && (last_char == ' '))
       {
-         cpd.column = next_tab_column(cpd.column);
+         int endcol = next_tab_column(cpd.column);
+         while (cpd.column < endcol)
+         {
+            add_char(' ');
+         }
+         return;
       }
       else
       {
-         cpd.column++;
+         fputc(ch, cpd.fout);
+         if (ch == '\t')
+         {
+            cpd.column = next_tab_column(cpd.column);
+         }
+         else
+         {
+            cpd.column++;
+         }
       }
    }
+   last_char = ch;
 }
 
 void add_text(const char *text)
@@ -345,6 +362,11 @@ static int calculate_comment_body_indent(const char *str, int len, int start_col
    int last_len  = 0;
    int width     = 0;
 
+   if (!cpd.settings[UO_cmt_indent_multi].b)
+   {
+      return(0);
+   }
+
    /* find the last line length */
    for (idx = len - 1; idx > 0; idx--)
    {
@@ -447,7 +469,6 @@ static_inline void add_spaces_after_star()
    }
 }
 
-
 static chunk_t *get_next_function(chunk_t *pc)
 {
    while ((pc = chunk_get_next(pc)) != NULL)
@@ -562,7 +583,6 @@ static void add_comment_javaparam(chunk_t *pc, cmt_reflow& cmt)
    }
 }
 
-
 /**
  * text starts with '$('. see if this matches a keyword and add text based
  * on that keyword.
@@ -632,6 +652,7 @@ static void add_comment_text(const char *text, int len,
                              cmt_reflow& cmt, bool esc_close)
 {
    bool was_star   = false;
+   bool was_slash  = false;
    bool was_dollar = false;
    bool in_word    = false;
 
@@ -661,17 +682,20 @@ static void add_comment_text(const char *text, int len,
       else
       {
          /* Escape a C closure in a CPP comment */
-         if (esc_close && was_star && (text[idx] == '/'))
+         if (esc_close &&
+             ((was_star && (text[idx] == '/')) ||
+              (was_slash && (text[idx] == '*'))))
          {
             add_char(' ');
          }
-         if (!in_word && !isspace(text[idx]))
+         if (!in_word && !unc_isspace(text[idx]))
          {
             cmt.word_count++;
          }
-         in_word = !isspace(text[idx]);
+         in_word = !unc_isspace(text[idx]);
          add_char(text[idx]);
          was_star   = (text[idx] == '*');
+         was_slash  = (text[idx] == '/');
          was_dollar = (text[idx] == '$');
       }
    }
@@ -756,6 +780,7 @@ static bool can_combine_comment(chunk_t *pc, cmt_reflow& cmt)
 static chunk_t *output_comment_c(chunk_t *first)
 {
    cmt_reflow cmt;
+
    output_cmt_start(cmt, first);
 
    cmt.cont_text = cpd.settings[UO_cmt_star_cont].b ? " *" : "  ";
@@ -800,6 +825,7 @@ static chunk_t *output_comment_c(chunk_t *first)
 static chunk_t *output_comment_cpp(chunk_t *first)
 {
    cmt_reflow cmt;
+
    output_cmt_start(cmt, first);
 
    /* CPP comments can't be grouped unless they are converted to C comments */
@@ -819,7 +845,7 @@ static chunk_t *output_comment_cpp(chunk_t *first)
    {
       /* nothing to group: just output a single line */
       add_text_len("/*", 2);
-      if (!isspace(first->str[2]))
+      if (!unc_isspace(first->str[2]))
       {
          add_char(' ');
       }
@@ -842,12 +868,12 @@ static chunk_t *output_comment_cpp(chunk_t *first)
    int offs;
    while (can_combine_comment(pc, cmt))
    {
-      offs = isspace(pc->str[2]) ? 1 : 0;
+      offs = unc_isspace(pc->str[2]) ? 1 : 0;
       add_comment_text(pc->str + 2 + offs, pc->len - (2 + offs), cmt, true);
       add_comment_text("\n", 1, cmt, false);
       pc = chunk_get_next(chunk_get_next(pc));
    }
-   offs = isspace(pc->str[2]) ? 1 : 0;
+   offs = unc_isspace(pc->str[2]) ? 1 : 0;
    add_comment_text(pc->str + 2 + offs, pc->len - (2 + offs), cmt, true);
    if (cpd.settings[UO_cmt_cpp_nl_end].b)
    {
@@ -880,8 +906,10 @@ void output_comment_multi(chunk_t *pc)
    bool       nl_end = false;
 
    cmt_reflow cmt;
+
    output_cmt_start(cmt, pc);
-   cmt.cont_text = cpd.settings[UO_cmt_star_cont].b ? " * " : "   ";
+   cmt.cont_text = !cpd.settings[UO_cmt_indent_multi].b ? "" :
+                   cpd.settings[UO_cmt_star_cont].b ? " * " : "   ";
 
    prev = chunk_get_prev(pc);
    if ((prev != NULL) && (prev->type != CT_NEWLINE))
@@ -893,12 +921,12 @@ void output_comment_multi(chunk_t *pc)
       col_diff = pc->orig_col - pc->column;
    }
 
-   //   fprintf(stderr, "Indenting1 line %d to col %d (orig=%d) col_diff=%d\n",
-   //           pc->orig_line, cmt_col, pc->orig_col, col_diff);
-
    xtra = calculate_comment_body_indent(pc->str, pc->len, pc->column);
 
-   ccol      = 1;
+   //LOG_FMT(LSYS, "Indenting1 line %d to col %d (orig=%d) col_diff=%d xtra=%d\n",
+   //        pc->orig_line, cmt_col, pc->orig_col, col_diff, xtra);
+
+   ccol      = pc->column;
    remaining = pc->len;
    cmt_str   = pc->str;
    line_len  = 0;
@@ -935,7 +963,7 @@ void output_comment_multi(chunk_t *pc)
          }
          else
          {
-            //fprintf(stderr, "%d] Text starts in col %d\n", line_count, ccol);
+            //LOG_FMT(LSYS, "%d] Text starts in col %d\n", line_count, ccol);
          }
       }
 
@@ -956,6 +984,19 @@ void output_comment_multi(chunk_t *pc)
             {
                line_len--;
             }
+            if ((line[line_len - 1] == '\\') && (line[line_len - 2] != '*'))
+            {
+               /* Kill off the backslash-newline */
+               line_len--;
+               while ((line_len > 0) &&
+                      ((line[line_len - 1] == ' ') ||
+                       (line[line_len - 1] == '\t')))
+               {
+                  line_len--;
+               }
+               line[line_len++] = ' ';
+               line[line_len++] = '\\';
+            }
             nl_end = true;
          }
          line[line_len] = 0;
@@ -969,7 +1010,7 @@ void output_comment_multi(chunk_t *pc)
             add_comment_text(line, line_len, cmt, false);
             if (nl_end)
             {
-               add_text_len("\n", 1);
+               add_char('\n');
             }
          }
          else
@@ -996,39 +1037,49 @@ void output_comment_multi(chunk_t *pc)
             }
             else
             {
-               add_spaces_before_star();
-
-               /* If this doesn't start with a '*' or '|' */
-               if ((line[0] != '*') && (line[0] != '|') && (line[0] != '#') &&
-                   (line[0] != '\\') && (line[0] != '+'))
+               /* If this doesn't start with a '*' or '|'.
+                * '\name' is a common parameter documentation thing.
+                */
+               if (cpd.settings[UO_cmt_indent_multi].b &&
+                   (line[0] != '*') && (line[0] != '|') && (line[0] != '#') &&
+                   ((line[0] != '\\') || unc_isalpha(line[1])) && (line[0] != '+'))
                {
+                  bool do_sp_add = false;
                   output_to_column(cmt_col, cpd.settings[UO_indent_with_tabs].n != 0);
+                  add_spaces_before_star();
+
                   if (cpd.settings[UO_cmt_star_cont].b)
                   {
                      cmt.cont_text = (xtra == 1) ? " * " : "*  ";
-                     add_spaces_after_star();
+                     do_sp_add = true;
                   }
                   else
                   {
                      cmt.cont_text = "   ";
                   }
                   add_text(cmt.cont_text);
+                  if (do_sp_add)
+                  {
+                     add_spaces_after_star();
+                  }
                   output_to_column(ccol, cpd.settings[UO_indent_with_tabs].n != 0);
                }
                else
                {
                   output_to_column(cmt_col + xtra, cpd.settings[UO_indent_with_tabs].n != 0);
+                  add_spaces_before_star();
+
                   int idx  = 0;
                   int sidx = 0;
                   if (xtra > 0)
                   {
                      lead[idx++] = ' ';
                   }
-                  while ((idx < 3) && !isspace(line[sidx]))
+                  while ((idx < 3) && !unc_isspace(line[sidx]))
                   {
                      lead[idx++] = line[sidx++];
                   }
-                  lead[idx] = 0;
+                  lead[idx]     = 0;
                   cmt.cont_text = lead;
                }
 
