@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: prototypes.h 984 2007-10-31 01:10:41Z bengardner $
+ * $Id: prototypes.h 1150 2008-03-06 04:30:12Z bengardner $
  */
 #ifndef C_PARSE_PROTOTYPES_H_INCLUDED
 #define C_PARSE_PROTOTYPES_H_INCLUDED
@@ -68,6 +68,7 @@ std::string op_val_to_string(argtype_e argtype, op_val_t op_val);
 void indent_text(void);
 void indent_preproc(void);
 void indent_to_column(chunk_t *pc, int column);
+void align_to_column(chunk_t *pc, int column);
 #define reindent_line(pc, col)   reindent_line2(pc, col, __func__, __LINE__)
 void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno);
 
@@ -79,11 +80,11 @@ void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno);
 void align_all(void);
 void align_backslash_newline(void);
 void align_right_comments(void);
-void align_func_proto(int span);
 void align_preprocessor(void);
 void align_struct_initializers(void);
 chunk_t *align_nl_cont(chunk_t *start);
 chunk_t *align_assign(chunk_t *first, int span, int thresh);
+void quick_align_again(void);
 
 
 /*
@@ -91,6 +92,14 @@ chunk_t *align_assign(chunk_t *first, int span, int thresh);
  */
 
 void do_braces(void);
+void add_long_closebrace_comment(void);
+
+
+/*
+ *  sorting.cpp
+ */
+
+void sort_imports(void);
 
 
 /*
@@ -141,8 +150,12 @@ void newlines_class_colon_pos(void);
 void newlines_cleanup_dup(void);
 void newline_after_multiline_comment(void);
 void do_blank_lines(void);
-chunk_t *newline_add_before(chunk_t *pc);
-chunk_t *newline_add_after(chunk_t *pc);
+
+#define newline_add_before(pc)   newline_add_before2(pc, __func__, __LINE__)
+chunk_t *newline_add_before2(chunk_t *pc, const char *fcn, int line);
+
+#define newline_add_after(pc)    newline_add_after2(pc, __func__, __LINE__)
+chunk_t *newline_add_after2(chunk_t *pc, const char *fcn, int line);
 
 
 /*
