@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: space.cpp 972 2007-10-17 21:54:41Z bengardner $
+ * $Id: space.cpp 1192 2008-06-06 19:12:54Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -15,7 +15,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 
 struct no_space_table_s
 {
@@ -32,14 +32,11 @@ struct no_space_table_s no_space_table[] =
 {
    { CT_INCDEC_BEFORE,  CT_WORD          },
    { CT_UNKNOWN,        CT_INCDEC_AFTER  },
-   { CT_UNKNOWN,        CT_ELLIPSIS      },
    { CT_UNKNOWN,        CT_LABEL_COLON   },
    { CT_UNKNOWN,        CT_PRIVATE_COLON },
    { CT_UNKNOWN,        CT_SEMICOLON     },
    { CT_UNKNOWN,        CT_D_TEMPLATE    },
    { CT_D_TEMPLATE,     CT_UNKNOWN       },
-   { CT_UNKNOWN,        CT_MEMBER        },
-   { CT_MEMBER,         CT_UNKNOWN       },
    { CT_MACRO_FUNC,     CT_FPAREN_OPEN   },
    { CT_PAREN_OPEN,     CT_UNKNOWN       },
    { CT_UNKNOWN,        CT_PAREN_CLOSE   },
@@ -242,6 +239,10 @@ argval_t do_space(chunk_t *first, chunk_t *second)
 
    if ((first->type == CT_ASSIGN) || (second->type == CT_ASSIGN))
    {
+      if (first->flags & PCF_IN_ENUM)
+      {
+         return(cpd.settings[UO_sp_enum_assign].a);
+      }
       return(cpd.settings[UO_sp_assign].a);
    }
 
@@ -295,6 +296,14 @@ argval_t do_space(chunk_t *first, chunk_t *second)
 
    if (second->type == CT_BYREF)
    {
+      if (cpd.settings[UO_sp_before_unnamed_byref].a != AV_IGNORE)
+      {
+         chunk_t *next = chunk_get_next_nc(second);
+         if ((next != NULL) && (next->type != CT_WORD))
+         {
+            return(cpd.settings[UO_sp_before_unnamed_byref].a);
+         }
+      }
       return(cpd.settings[UO_sp_before_byref].a);
    }
 
@@ -333,6 +342,11 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(AV_FORCE);
    }
 
+   if ((first->type == CT_BRACE_OPEN) && (second->type == CT_BRACE_CLOSE))
+   {
+      return(cpd.settings[UO_sp_inside_braces_empty].a);
+   }
+
    if (second->type == CT_BRACE_CLOSE)
    {
       if (second->parent_type == CT_ENUM)
@@ -367,9 +381,9 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(AV_REMOVE);
    }
 
-   if (second->type == CT_MEMBER)
+   if ((first->type == CT_MEMBER) || (second->type == CT_MEMBER))
    {
-      return(AV_REMOVE);
+      return(cpd.settings[UO_sp_member].a);
    }
 
    if (first->type == CT_C99_MEMBER)
@@ -464,6 +478,13 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_inside_fparen].a);
    }
 
+   if ((first->type == CT_PAREN_CLOSE) &&
+       ((first->parent_type == CT_OC_MSG_SPEC) ||
+        (first->parent_type == CT_OC_MSG_DECL)))
+   {
+      return(cpd.settings[UO_sp_after_oc_type].a);
+   }
+
    /* "(a + 3)" vs "( a + 3 )" */
    if (first->type == CT_PAREN_OPEN)
    {
@@ -521,6 +542,17 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_before_comma].a);
    }
 
+   if ((cpd.settings[UO_sp_after_class_colon].a != AV_IGNORE) &&
+       (first->type == CT_CLASS_COLON))
+   {
+      return(cpd.settings[UO_sp_after_class_colon].a);
+   }
+   if ((cpd.settings[UO_sp_before_class_colon].a != AV_IGNORE) &&
+       (second->type == CT_CLASS_COLON))
+   {
+      return(cpd.settings[UO_sp_before_class_colon].a);
+   }
+
    if (first->type == CT_DOT)
    {
       return(AV_REMOVE);
@@ -569,10 +601,20 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_after_ptr_star].a);
    }
 
-   if ((second->type == CT_PTR_TYPE) &&
-       (cpd.settings[UO_sp_before_ptr_star].a != AV_IGNORE))
+   if (second->type == CT_PTR_TYPE)
    {
-      return(cpd.settings[UO_sp_before_ptr_star].a);
+      if (cpd.settings[UO_sp_before_unnamed_ptr_star].a != AV_IGNORE)
+      {
+         chunk_t *next = chunk_get_next_nc(second);
+         if ((next != NULL) && (next->type != CT_WORD))
+         {
+            return(cpd.settings[UO_sp_before_unnamed_ptr_star].a);
+         }
+      }
+      if (cpd.settings[UO_sp_before_ptr_star].a != AV_IGNORE)
+      {
+         return(cpd.settings[UO_sp_before_ptr_star].a);
+      }
    }
 
    if (first->type == CT_OPERATOR)
@@ -657,7 +699,8 @@ argval_t do_space(chunk_t *first, chunk_t *second)
    if ((second->type != CT_PTR_TYPE) &&
        ((first->type == CT_QUALIFIER) || (first->type == CT_TYPE)))
    {
-      return(AV_FORCE);
+      arg = cpd.settings[UO_sp_after_type].a;
+      return((arg != AV_REMOVE) ? arg : AV_FORCE);
    }
 
    if (first->type == CT_COMMENT)
@@ -698,6 +741,22 @@ argval_t do_space(chunk_t *first, chunk_t *second)
    {
       return(cpd.settings[UO_sp_deref].a);
    }
+   if ((first->type == CT_POS) || (first->type == CT_NEG))
+   {
+      return(cpd.settings[UO_sp_sign].a);
+   }
+   if ((first->type == CT_INCDEC_BEFORE) || (second->type == CT_INCDEC_AFTER))
+   {
+      return(cpd.settings[UO_sp_incdec].a);
+   }
+   if (first->type == CT_OC_SCOPE)
+   {
+      return(cpd.settings[UO_sp_after_oc_scope].a);
+   }
+   if (second->type == CT_OC_COLON)
+   {
+      return(cpd.settings[UO_sp_before_oc_colon].a);
+   }
 
    for (idx = 0; idx < (int)ARRAY_SIZE(no_space_table); idx++)
    {
@@ -806,6 +865,12 @@ void space_text(void)
          }
 
          int av = do_space(pc, next);
+
+         LOG_FMT(LSPACE, "%s: [%s] %.*s <==> [%s] %.*s = %d\n",
+                 __func__,
+                 get_token_name(pc->type), pc->len, pc->str,
+                 get_token_name(next->type), next->len, next->str, av);
+
          if (pc->flags & PCF_FORCE_SPACE)
          {
             av |= AV_ADD;
@@ -928,9 +993,22 @@ int space_col_align(chunk_t *first, chunk_t *second)
    av = do_space(first, second);
 
    coldiff = first->len;
-   if (av != AV_REMOVE)
+   switch (av)
    {
+   case AV_ADD:
+   case AV_FORCE:
       coldiff++;
+      break;
+
+   case AV_REMOVE:
+      break;
+
+   case AV_IGNORE:
+      if (second->orig_col> (first->orig_col + first->len))
+      {
+         coldiff++;
+      }
+      break;
    }
    return(coldiff);
 }
