@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: token_enum.h 946 2007-10-01 17:39:35Z bengardner $
+ * $Id: token_enum.h 1169 2008-05-02 01:19:53Z bengardner $
  */
 #ifndef TOKEN_ENUM_H_INCLUDED
 #define TOKEN_ENUM_H_INCLUDED
@@ -68,6 +68,7 @@ typedef enum
    CT_TEMPLATE,            /* template<...> */
 
    CT_ASSIGN,              /* =, +=, /=, etc */
+   CT_ASSIGN_NL,           /* Assign followed by a newline - fake item for indenting */
    CT_SASSIGN,             /* 'and_eq' */
    CT_COMPARE,             /* ==, !=, <=, >= */
    CT_SCOMPARE,            /* compare op that is a string 'is', 'neq' */
@@ -128,6 +129,8 @@ typedef enum
    CT_THROW,
    CT_TRY,
    CT_USING,
+   CT_D_WITH,           /* D: paren+braced */
+   CT_D_MODULE,
    CT_SUPER,
    CT_DELEGATE,
    CT_BODY,
@@ -181,6 +184,7 @@ typedef enum
    CT_MACRO_FUNC,       /* function-like macro */
    CT_MACRO,            /* a macro def */
    CT_QUALIFIER,        /* static, const, etc */
+   CT_EXTERN,           /* extern */
    CT_ALIGN,            /* paren'd qualifier: align(4) struct a { } */
    CT_TYPE,
    CT_PTR_TYPE,         /* a '*' as part of a type */
@@ -191,6 +195,10 @@ typedef enum
    CT_OC_IMPL,          /* ObjC: @implementation */
    CT_OC_INTF,          /* ObjC: @interface */
    CT_OC_CLASS,         /* ObjC: the name after @interface or @implementation */
+   CT_OC_SCOPE,         /* ObjC: the '-' or '+' in '-(void) func: (int) i;' */
+   CT_OC_MSG_SPEC,      /* ObjC: msg spec '-(void) func: (int) i;' */
+   CT_OC_MSG_DECL,      /* ObjC: msg decl '-(void) func: (int) i { }' */
+   CT_OC_COLON,         /* ObjC: the colon in a msg spec */
 
    /* start PP types */
    CT_PP_DEFINE,        /* #define */
