@@ -7,7 +7,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: tokenize.cpp 971 2007-10-17 21:26:22Z bengardner $
+ * $Id: tokenize.cpp 1088 2008-01-30 01:24:42Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -18,7 +18,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 
 
 static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape);
@@ -194,8 +194,8 @@ static bool parse_number(chunk_t *pc)
    bool did_hex = false;
 
    /* A number must start with a digit or a dot, followed by a digit */
-   if (!isdigit(pc->str[0]) &&
-       ((pc->str[0] != '.') || !isdigit(pc->str[1])))
+   if (!unc_isdigit(pc->str[0]) &&
+       ((pc->str[0] != '.') || !unc_isdigit(pc->str[1])))
    {
       return(false);
    }
@@ -212,14 +212,14 @@ static bool parse_number(chunk_t *pc)
     */
    if (pc->str[0] == '0')
    {
-      switch (toupper(pc->str[1]))
+      switch (unc_toupper(pc->str[1]))
       {
       case 'X':               /* hex */
          did_hex = true;
          do
          {
             len++;
-         } while (isxdigit(pc->str[len]) || (pc->str[len] == '_'));
+         } while (unc_isxdigit(pc->str[len]) || (pc->str[len] == '_'));
          break;
 
       case 'B':               /* binary */
@@ -253,7 +253,7 @@ static bool parse_number(chunk_t *pc)
    else
    {
       /* Regular int or float */
-      while (isdigit(pc->str[len]) || (pc->str[len] == '_'))
+      while (unc_isdigit(pc->str[len]) || (pc->str[len] == '_'))
       {
          len++;
       }
@@ -266,14 +266,14 @@ static bool parse_number(chunk_t *pc)
       is_float = true;
       if (did_hex)
       {
-         while (isxdigit(pc->str[len]) || (pc->str[len] == '_'))
+         while (unc_isxdigit(pc->str[len]) || (pc->str[len] == '_'))
          {
             len++;
          }
       }
       else
       {
-         while (isdigit(pc->str[len]) || (pc->str[len] == '_'))
+         while (unc_isdigit(pc->str[len]) || (pc->str[len] == '_'))
          {
             len++;
          }
@@ -285,7 +285,7 @@ static bool parse_number(chunk_t *pc)
     * C/C++/D/Java: eEpP
     * C#/Pawn:      eE
     */
-   tmp = toupper(pc->str[len]);
+   tmp = unc_toupper(pc->str[len]);
    if ((tmp == 'E') || (tmp == 'P'))
    {
       is_float = true;
@@ -294,7 +294,7 @@ static bool parse_number(chunk_t *pc)
       {
          len++;
       }
-      while (isdigit(pc->str[len]) || (pc->str[len] == '_'))
+      while (unc_isdigit(pc->str[len]) || (pc->str[len] == '_'))
       {
          len++;
       }
@@ -313,7 +313,7 @@ static bool parse_number(chunk_t *pc)
     */
    while (1)
    {
-      tmp = toupper(pc->str[len]);
+      tmp = unc_toupper(pc->str[len]);
       if ((tmp == 'I') || (tmp == 'F') || (tmp == 'D') || (tmp == 'M'))
       {
          is_float = true;
@@ -557,7 +557,7 @@ static bool parse_bs_newline(chunk_t *pc)
 {
    pc->len = 1;
 
-   while (isspace(pc->str[pc->len]))
+   while (unc_isspace(pc->str[pc->len]))
    {
       if ((pc->str[pc->len] == '\r') || (pc->str[pc->len] == '\n'))
       {
@@ -691,6 +691,16 @@ static bool parse_next(chunk_t *pc)
       }
    }
 
+   /* Check for Obj-C NSString constants, ie @"hello" */
+   if (((cpd.lang_flags & LANG_OC) != 0) && (*pc->str == '@'))
+   {
+      if (pc->str[1] == '"')
+      {
+         parse_string(pc, 1, true);
+         return(true);
+      }
+   }
+
    /* PAWN specific stuff */
    if ((cpd.lang_flags & LANG_PAWN) != 0)
    {
@@ -741,6 +751,16 @@ static bool parse_next(chunk_t *pc)
          parse_string(pc, (*pc->str == 'L') ? 1 : 0, true);
          return(true);
       }
+
+      if ((*pc->str == '<') && (cpd.in_preproc == CT_PP_DEFINE))
+      {
+         if (chunk_get_tail()->type == CT_MACRO)
+         {
+            /* We have "#define XXX <", assume '<' starts an include string */
+            parse_string(pc, 0, false);
+            return(true);
+         }
+      }
    }
 
    /* Check for pawn/ObjectiveC identifiers */
