@@ -8,14 +8,14 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: tokenize_cleanup.cpp 942 2007-09-27 02:14:29Z bengardner $
+ * $Id: tokenize_cleanup.cpp 1169 2008-05-02 01:19:53Z bengardner $
  */
 
 #include "uncrustify_types.h"
 #include "prototypes.h"
 #include "chunk_list.h"
 #include "char_table.h"
-#include <cctype>
+#include "unc_ctype.h"
 #include <cstring>
 
 static void check_template(chunk_t *start);
@@ -87,6 +87,30 @@ void tokenize_cleanup(void)
          }
       }
 
+      /* change extern to qualifier if extern isn't followed by a string or
+       * an open paren
+       */
+      if (pc->type == CT_EXTERN)
+      {
+         if (next->type == CT_STRING)
+         {
+            /* Probably 'extern "C"' */
+         }
+         else if (next->type == CT_PAREN_OPEN)
+         {
+            /* Probably 'extern (C)' */
+         }
+         else
+         {
+            /* Something else followed by a open brace */
+            tmp = chunk_get_next_ncnl(next);
+            if ((tmp != NULL) || (tmp->type != CT_BRACE_OPEN))
+            {
+               pc->type = CT_QUALIFIER;
+            }
+         }
+      }
+
       /**
        * Change CT_STAR to CT_PTR_TYPE if preceeded by CT_TYPE,
        * CT_QUALIFIER, or CT_PTR_TYPE.
@@ -161,17 +185,6 @@ void tokenize_cleanup(void)
          pc->type = CT_WORD;
       }
 
-      /* REVISIT: This duplicates above logic */
-      // if ((pc->type == CT_ENUM) ||
-      //     (pc->type == CT_STRUCT) ||
-      //     (pc->type == CT_UNION))
-      // {
-      //    if (get_char_table(*next->str) & CT_KW1)
-      //    {
-      //       next->type = CT_TYPE;
-      //    }
-      // }
-
       /* Change item after operator (>=, ==, etc) to a CT_FUNCTION */
       if (pc->type == CT_OPERATOR)
       {
@@ -191,32 +204,44 @@ void tokenize_cleanup(void)
          }
          else
          {
-            /* Mark chunks between 'operator' and '('.
-             * If 'next' is a WORD, then the last 'type' present
-             * is the function name. Otherwise, the item after the
-             * 'operator' is the function name.
-             */
-            tmp2 = next;
-            if ((next->flags & PCF_PUNCTUATOR) == 0)
+            /* Scan until we hit a '(' or ';' */
+            tmp = next;
+            while ((tmp != NULL) &&
+                   (tmp->type != CT_PAREN_OPEN) &&
+                   (tmp->type != CT_SEMICOLON))
             {
-               tmp  = chunk_get_next_ncnl(next);
-               while ((tmp != NULL) && (tmp->type != CT_PAREN_OPEN))
+               tmp = chunk_get_next_ncnl(tmp);
+            }
+
+            if ((tmp != NULL) && (tmp->type == CT_PAREN_OPEN))
+            {
+               /* Mark chunks between 'operator' and '('.
+                * If 'next' is a WORD, then the last 'type' present
+                * is the function name. Otherwise, the item after the
+                * 'operator' is the function name.
+                */
+               tmp2 = next;
+               if ((next->flags & PCF_PUNCTUATOR) == 0)
                {
-                  tmp->parent_type = CT_OPERATOR;
-                  make_type(tmp);
-                  if (tmp->type == CT_TYPE)
+                  tmp = chunk_get_next_ncnl(next);
+                  while ((tmp != NULL) && (tmp->type != CT_PAREN_OPEN))
                   {
-                     tmp2 = tmp;
+                     tmp->parent_type = CT_OPERATOR;
+                     make_type(tmp);
+                     if (tmp->type == CT_TYPE)
+                     {
+                        tmp2 = tmp;
+                     }
+                     tmp = chunk_get_next_ncnl(tmp);
                   }
-                  tmp = chunk_get_next_ncnl(tmp);
-               }
-               if (tmp2->type != CT_TYPE)
-               {
-                  tmp2 = next;
+                  if (tmp2->type != CT_TYPE)
+                  {
+                     tmp2 = next;
+                  }
                }
+               tmp2->type        = CT_FUNCTION;
+               tmp2->parent_type = CT_OPERATOR;
             }
-            tmp2->type        = CT_FUNCTION;
-            tmp2->parent_type = CT_OPERATOR;
          }
          if (chunk_is_addr(prev))
          {
@@ -289,7 +314,7 @@ void tokenize_cleanup(void)
       /* Detect Objective C class name */
       if ((pc->type == CT_OC_IMPL) || (pc->type == CT_OC_INTF))
       {
-         next->type = CT_CLASS;
+         next->type = CT_OC_CLASS;
          tmp        = chunk_get_next_ncnl(next);
          if (tmp != NULL)
          {
