@@ -6,7 +6,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: uncrustify_types.h 979 2007-10-25 02:11:52Z bengardner $
+ * $Id: uncrustify_types.h 1175 2008-05-09 20:36:54Z bengardner $
  */
 
 #ifndef UNCRUSTIFY_TYPES_H_INCLUDED
@@ -20,6 +20,9 @@
 #include "logger.h"
 #include <cstdio>
 #include <assert.h>
+#ifdef HAVE_UTIME_H
+#include <utime.h>
+#endif
 
 
 /**
@@ -64,6 +67,7 @@ struct paren_stack_entry
 /* TODO: put this on a linked list */
 struct parse_frame
 {
+   int                      ref_no;
    int                      level;           // level of parens/square/angle/brace
    int                      brace_level;     // level of brace/vbrace
    int                      pp_level;        // level of preproc #if stuff
@@ -74,7 +78,7 @@ struct parse_frame
    int                      pse_tos;
    int                      paren_count;
 
-   int                      in_ifdef;
+   c_token_t                in_ifdef;
    int                      stmt_count;
    int                      expr_count;
 
@@ -112,10 +116,10 @@ struct parse_frame
 #define PCF_ANCHOR             (1 << 25)  /* aligning anchor */
 #define PCF_PUNCTUATOR         (1 << 26)
 #define PCF_INSERTED           (1 << 27)  /* chunk was inserted from another file */
-#define PCF_TBD_28             (1 << 28)
+#define PCF_ALIGN_START        (1 << 28)
 #define PCF_TBD_29             (1 << 29)
 #define PCF_TBD_30             (1 << 30)
-#define PCF_TBD_31             (1 << 31)
+#define PCF_OWN_STR            (1 << 31)  /* chunk owns the memory at str */
 
 
 /* flags that get copied when a new chunk is inserted */
@@ -155,34 +159,44 @@ static const char *pcf_names[] =
    "ANCHOR",
    "PUNCTUATOR",
    "INSERTED",
-   "TBD_28",
+   "ALIGN_START",
    "TBD_29",
    "TBD_30",
-   "TBD_31",
+   "OWN_STR",
 };
 #endif
 
+struct align_ptr_t
+{
+   chunk_t *next; /* NULL or the chunk that should be under this one */
+   bool    right_align;
+   int     star_style;
+   int     amp_style;
+   int     gap;
+};
+
 /** This is the main type of this program */
 struct chunk_t
 {
-   chunk_t    *next;
-   chunk_t    *prev;
-   c_token_t  type;
-   c_token_t  parent_type;     /* usually CT_NONE */
-   UINT32     orig_line;
-   UINT32     orig_col;
-   UINT32     orig_col_end;
-   UINT32     flags;            /* see PCF_xxx */
-   int        column;           /* column of chunk */
-   int        column_indent;    /* if 1st on a line, set to the 'indent'
-                                 * column, which may be less that the real column */
-   int        nl_count;         /* number of newlines in CT_NEWLINE */
-   int        level;            /* nest level in {, (, or [ */
-   int        brace_level;      /* nest level in braces only */
-   int        pp_level;         /* nest level in #if stuff */
-   bool       after_tab;
-   int        len;
-   const char *str;
+   chunk_t     *next;
+   chunk_t     *prev;
+   align_ptr_t align;
+   c_token_t   type;
+   c_token_t   parent_type;     /* usually CT_NONE */
+   UINT32      orig_line;
+   UINT32      orig_col;
+   UINT32      orig_col_end;
+   UINT32      flags;            /* see PCF_xxx */
+   int         column;           /* column of chunk */
+   int         column_indent;    /* if 1st on a line, set to the 'indent'
+                                  * column, which may be less that the real column */
+   int         nl_count;         /* number of newlines in CT_NEWLINE */
+   int         level;            /* nest level in {, (, or [ */
+   int         brace_level;      /* nest level in braces only */
+   int         pp_level;         /* nest level in #if stuff */
+   bool        after_tab;
+   int         len;
+   const char  *str;
 };
 
 enum
@@ -260,8 +274,11 @@ typedef struct chunk_stack
 
 struct file_mem
 {
-   char *data;
-   int  length;
+   char           *data;
+   int            length;
+#ifdef HAVE_UTIME_H
+   struct utimbuf utb;
+#endif
 };
 
 struct cp_data
@@ -277,7 +294,7 @@ struct cp_data
 
    int                lang_flags; // LANG_xxx
 
-   UINT16             line_number;
+   UINT32             line_number;
    UINT16             column;  /* column for parsing */
 
    /* stuff to auto-detect line endings */
